@@ -8,6 +8,7 @@
 # Standard library imports
 import os
 import os.path as osp
+import random
 import sys
 import threading
 import traceback
@@ -23,16 +24,19 @@ import psutil
 import pytest
 
 # Spyder imports
+from spyder.api.plugin_registration.registry import PLUGIN_REGISTRY
+from spyder.api.plugins import Plugins
 from spyder.app import start
 from spyder.config.base import get_home_dir, running_in_ci
 from spyder.config.manager import CONF
 from spyder.plugins.ipythonconsole.utils.kernelspec import SpyderKernelSpec
 from spyder.plugins.projects.api import EmptyProject
+from spyder.plugins.run.api import RunActions, StoredRunConfigurationExecutor
+from spyder.plugins.toolbar.api import ApplicationToolbars
 from spyder.utils import encoding
 from spyder.utils.environ import (get_user_env, set_user_env,
                                   amend_user_shell_init)
 
-
 # =============================================================================
 # ---- Constants
 # =============================================================================
@@ -220,14 +224,15 @@ def preferences_dialog_helper(qtbot, main_window, section):
     # Wait until the window is fully up
     shell = main_window.ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(
-        lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
+        lambda: shell.spyder_kernel_ready and shell._prompt_html is not None,
+        timeout=SHELL_TIMEOUT
+    )
 
     main_window.show_preferences()
     preferences = main_window.preferences
     container = preferences.get_container()
 
-    qtbot.waitUntil(lambda: container.dialog is not None,
-                    timeout=5000)
+    qtbot.waitUntil(lambda: container.dialog is not None, timeout=5000)
     dlg = container.dialog
     index = dlg.get_index_by_name(section)
     page = dlg.get_page(index)
@@ -235,6 +240,39 @@ def preferences_dialog_helper(qtbot, main_window, section):
     return dlg, index, page
 
 
+def generate_run_parameters(mainwindow, filename, selected=None,
+                            executor=None):
+    """Generate run configuration parameters for a given filename."""
+    file_uuid = mainwindow.editor.get_widget().id_per_file[filename]
+    if executor is None:
+        executor = mainwindow.ipyconsole.NAME
+
+    file_run_params = StoredRunConfigurationExecutor(
+        executor=executor,
+        selected=selected,
+    )
+
+    return {file_uuid: file_run_params}
+
+
+def get_random_dockable_plugin(main_window, exclude=None):
+    """Get a random dockable plugin and give it focus."""
+    plugins = main_window.get_dockable_plugins()
+    for plugin_name, plugin in plugins:
+        if exclude and plugin_name in exclude:
+            plugins.remove((plugin_name, plugin))
+
+    plugin = random.choice(plugins)[1]
+
+    if not plugin.get_widget().toggle_view_action.isChecked():
+        plugin.toggle_view(True)
+        plugin._hide_after_test = True
+
+    plugin.switch_to_plugin()
+    plugin.get_widget().get_focus_widget().setFocus()
+    return plugin
+
+
 # =============================================================================
 # ---- Pytest hooks
 # =============================================================================
@@ -261,10 +299,19 @@ def cleanup(request, qapp):
         if hasattr(main_window, 'window') and main_window.window is not None:
             window = main_window.window
             main_window.window = None
+            window.closing(close_immediately=True)
             window.close()
             window = None
             CONF.reset_to_defaults(notification=False)
+            CONF.reset_manager()
+            PLUGIN_REGISTRY.reset()
+
         if qapp.instance():
+            for widget in qapp.allWidgets():
+                try:
+                    widget.close()
+                except RuntimeError:
+                    pass
             qapp.quit()
 
     request.addfinalizer(close_window)
@@ -278,33 +325,31 @@ def main_window(request, tmpdir, qtbot):
     # fails
     super_processEvents = QApplication.processEvents
 
-    # Disable Kite provider
-    CONF.set('completions', 'enabled_providers', {'kite': False})
-
     # Don't show tours message
     CONF.set('tours', 'show_tour_message', False)
 
     # Tests assume inline backend
-    CONF.set('ipython_console', 'pylab/backend', 0)
+    CONF.set('ipython_console', 'pylab/backend', 'inline')
 
     # Test assume the plots are rendered in the console as png
     CONF.set('plots', 'mute_inline_plotting', False)
-    CONF.set('ipython_console', 'pylab/inline/figure_format', 0)
+    CONF.set('ipython_console', 'pylab/inline/figure_format', "png")
 
     # Set exclamation mark to True
-    CONF.set('ipython_console', 'pdb_use_exclamation_mark', True)
+    CONF.set('debugger', 'pdb_use_exclamation_mark', True)
 
     # Check if we need to use introspection in a given test
     # (it's faster and less memory consuming not to use it!)
     use_introspection = request.node.get_closest_marker('use_introspection')
 
     if use_introspection:
-        os.environ['SPY_TEST_USE_INTROSPECTION'] = 'True'
+        CONF.set('completions', ('enabled_providers', 'lsp'), True)
+        CONF.set('completions', ('enabled_providers', 'fallback'), True)
+        CONF.set('completions', ('enabled_providers', 'snippets'), True)
     else:
-        try:
-            os.environ.pop('SPY_TEST_USE_INTROSPECTION')
-        except KeyError:
-            pass
+        CONF.set('completions', ('enabled_providers', 'lsp'), False)
+        CONF.set('completions', ('enabled_providers', 'fallback'), False)
+        CONF.set('completions', ('enabled_providers', 'snippets'), False)
 
     # Only use single_instance mode for tests that require it
     single_instance = request.node.get_closest_marker('single_instance')
@@ -353,9 +398,6 @@ def main_window(request, tmpdir, qtbot):
     QApplication.processEvents()
 
     if not hasattr(main_window, 'window') or main_window.window is None:
-        from spyder.api.plugin_registration.registry import PLUGIN_REGISTRY
-        PLUGIN_REGISTRY.reset()
-
         # Start the window
         window = start.main()
         main_window.window = window
@@ -368,6 +410,35 @@ def main_window(request, tmpdir, qtbot):
             # even if the same mainwindow instance is reused
             window.ipyconsole.create_new_client(give_focus=True)
 
+    # Add a handle to the "Debug file" button to access it quickly because
+    # it's used a lot.
+    toolbar = window.get_plugin(Plugins.Toolbar)
+    debug_toolbar = toolbar.get_application_toolbar(ApplicationToolbars.Debug)
+    debug_action = window.run.get_action(
+        "run file in debugger")
+    debug_button = debug_toolbar.widgetForAction(debug_action)
+    window.debug_button = debug_button
+
+    # Add a handle to the run buttons to access it quickly because they are
+    # used a lot.
+    run_toolbar = toolbar.get_application_toolbar(ApplicationToolbars.Run)
+    run_action = window.run.get_action(RunActions.Run)
+    run_button = run_toolbar.widgetForAction(run_action)
+    window.run_button = run_button
+
+    run_cell_action = window.run.get_action('run cell')
+    run_cell_button = run_toolbar.widgetForAction(run_cell_action)
+    window.run_cell_button = run_cell_button
+
+    run_cell_and_advance_action = window.run.get_action('run cell and advance')
+    run_cell_and_advance_button = run_toolbar.widgetForAction(
+        run_cell_and_advance_action)
+    window.run_cell_and_advance_button = run_cell_and_advance_button
+
+    run_selection_action = window.run.get_action('run selection and advance')
+    run_selection_button = run_toolbar.widgetForAction(run_selection_action)
+    window.run_selection_button = run_selection_button
+
     QApplication.processEvents()
 
     if os.name != 'nt':
@@ -398,37 +469,48 @@ def main_window(request, tmpdir, qtbot):
                 print('info_page')
                 print(client.info_page)
             main_window.window = None
+            window.closing(close_immediately=True)
             window.close()
             window = None
             CONF.reset_to_defaults(notification=False)
+            CONF.reset_manager()
+            PLUGIN_REGISTRY.reset()
+
         else:
             # Try to close used mainwindow directly on fixture
             # after running test that uses the fixture
             # Currently 'test_out_runfile_runcell' is the last tests so
             # in order to prevent errors finalizing the test suit such test has
-            # this marker
+            # this marker.
+            # Also, try to decrease chances of freezes/timeouts from tests that
+            # are known to have leaks by also closing the main window for them.
+            known_leak = request.node.get_closest_marker(
+                'known_leak')
             close_main_window = request.node.get_closest_marker(
                 'close_main_window')
-            if close_main_window:
+            if close_main_window or known_leak:
                 main_window.window = None
+                window.closing(close_immediately=True)
                 window.close()
                 window = None
                 CONF.reset_to_defaults(notification=False)
+                CONF.reset_manager()
+                PLUGIN_REGISTRY.reset()
             else:
                 try:
-                    # Close everything we can think of
-                    window.switcher.close()
+                    # Close or hide everything we can think of
+                    window.switcher.hide()
+                    window.switcher.on_close()
 
                     # Close editor related elements
                     window.editor.close_all_files()
-                    # force close all files
-                    while window.editor.editorstacks[0].close_file(force=True):
+
+                    # Force close all files
+                    editor_widget = window.editor.get_widget()
+                    while editor_widget.editorstacks[0].close_file(force=True):
                         pass
-                    for editorwindow in window.editor.editorwindows:
+                    for editorwindow in editor_widget.editorwindows:
                         editorwindow.close()
-                    editorstack = window.editor.get_current_editorstack()
-                    if editorstack.switcher_dlg:
-                        editorstack.switcher_dlg.close()
 
                     window.projects.close_project()
 
@@ -452,24 +534,20 @@ def main_window(request, tmpdir, qtbot):
                     (window.ipyconsole.get_widget()
                         .create_new_client_if_empty) = False
                     window.ipyconsole.restart()
-
                 except Exception:
                     main_window.window = None
+                    window.closing(close_immediately=True)
                     window.close()
                     window = None
                     CONF.reset_to_defaults(notification=False)
+                    CONF.reset_manager()
+                    PLUGIN_REGISTRY.reset()
                     return
 
                 if os.name == 'nt':
                     # Do not test leaks on windows
                     return
 
-                known_leak = request.node.get_closest_marker(
-                    'known_leak')
-                if known_leak:
-                    # This test has a known leak
-                    return
-
                 def show_diff(init_list, now_list, name):
                     sys.stderr.write(f"Extra {name} before test:\n")
                     for item in init_list:
@@ -498,15 +576,18 @@ def main_window(request, tmpdir, qtbot):
                     show_diff(init_threads, threads, "thread")
                     sys.stderr.write("Running Threads stacks:\n")
                     now_thread_ids = [t.ident for t in now_threads]
-                    for threadId, frame in sys._current_frames().items():
-                        if threadId in now_thread_ids:
+                    for thread_id, frame in sys._current_frames().items():
+                        if thread_id in now_thread_ids:
                             sys.stderr.write(
                                 "\nThread " + str(threads) + ":\n")
                             traceback.print_stack(frame)
                     main_window.window = None
+                    window.closing(close_immediately=True)
                     window.close()
                     window = None
                     CONF.reset_to_defaults(notification=False)
+                    CONF.reset_manager()
+                    PLUGIN_REGISTRY.reset()
                     raise
 
                 try:
@@ -517,9 +598,12 @@ def main_window(request, tmpdir, qtbot):
                     subprocesses = [repr(f) for f in proc.children()]
                     show_diff(init_subprocesses, subprocesses, "processes")
                     main_window.window = None
+                    window.closing(close_immediately=True)
                     window.close()
                     window = None
                     CONF.reset_to_defaults(notification=False)
+                    CONF.reset_manager()
+                    PLUGIN_REGISTRY.reset()
                     raise
 
                 try:
@@ -533,9 +617,12 @@ def main_window(request, tmpdir, qtbot):
                 except Exception:
                     show_diff(init_files, files, "files")
                     main_window.window = None
+                    window.closing(close_immediately=True)
                     window.close()
                     window = None
                     CONF.reset_to_defaults(notification=False)
+                    CONF.reset_manager()
+                    PLUGIN_REGISTRY.reset()
                     raise
 
 
