@@ -6,28 +6,50 @@
 
 """Appearance entry in Preferences."""
 
+import configparser
+import sys
+
+import qstylizer.style
 from qtconsole.styles import dark_color
-from qtpy.QtCore import Slot
-from qtpy.QtWidgets import (QFontComboBox, QGridLayout, QGroupBox, QMessageBox,
-                            QPushButton, QStackedWidget, QVBoxLayout)
+from qtpy.QtCore import Qt, Slot
+from qtpy.QtGui import QFont
+from qtpy.QtWidgets import (
+    QFontComboBox,
+    QGridLayout,
+    QGroupBox,
+    QLabel,
+    QMessageBox,
+    QPushButton,
+    QStackedWidget,
+    QVBoxLayout,
+)
 
 from spyder.api.preferences import PluginConfigPage
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.config.gui import get_font, is_dark_font_color, set_font
 from spyder.config.manager import CONF
 from spyder.plugins.appearance.widgets import SchemeEditor
 from spyder.utils import syntaxhighlighters
-from spyder.utils.palette import QStylePalette
+from spyder.utils.palette import SpyderPalette
+from spyder.utils.stylesheet import AppStyle
 from spyder.widgets.simplecodeeditor import SimpleCodeEditor
 
-# Localization
-_ = get_translation('spyder')
+
+PREVIEW_TEXT = (
+    '"""A string"""\n\n'
+    '# A comment\n\n'
+    'class Foo(object):\n'
+    '    def __init__(self):\n'
+    '        bar = 42\n'
+    '        print(bar)\n'
+)
 
 
 class AppearanceConfigPage(PluginConfigPage):
 
     def __init__(self, plugin, parent):
         super().__init__(plugin, parent)
+        self._is_shown = False
         self.pre_apply_callback = self.check_color_scheme_notification
 
         # Notifications for this option are disabled when the plugin is
@@ -59,16 +81,10 @@ class AppearanceConfigPage(PluginConfigPage):
         )
         self.ui_combobox = ui_theme_combo.combobox
 
-        themes = ['Spyder 2', 'Spyder 3']
-        icon_choices = list(zip(themes, [theme.lower() for theme in themes]))
-        icons_combo = self.create_combobox(_('Icon theme'), icon_choices,
-                                           'icon_theme', restart=True)
 
         theme_comboboxes_layout = QGridLayout()
         theme_comboboxes_layout.addWidget(ui_theme_combo.label, 0, 0)
         theme_comboboxes_layout.addWidget(ui_theme_combo.combobox, 0, 1)
-        theme_comboboxes_layout.addWidget(icons_combo.label, 1, 0)
-        theme_comboboxes_layout.addWidget(icons_combo.combobox, 1, 1)
 
         theme_layout = QVBoxLayout()
         theme_layout.addLayout(theme_comboboxes_layout)
@@ -83,18 +99,24 @@ class AppearanceConfigPage(PluginConfigPage):
         self.delete_button = QPushButton(_("Delete scheme"))
         self.reset_button = QPushButton(_("Reset to defaults"))
 
-        self.preview_editor = SimpleCodeEditor(self)
         self.stacked_widget = QStackedWidget(self)
-        self.scheme_editor_dialog = SchemeEditor(parent=self,
-                                                 stack=self.stacked_widget)
+        self.scheme_editor_dialog = SchemeEditor(
+            parent=self,
+            stack=self.stacked_widget
+        )
 
         self.scheme_choices_dict = {}
-        schemes_combobox_widget = self.create_combobox('', [('', '')],
-                                                       'selected')
+        schemes_combobox_widget = self.create_combobox(
+            '', [('', '')], 'selected', items_elide_mode=Qt.ElideNone
+        )
         self.schemes_combobox = schemes_combobox_widget.combobox
 
         # Syntax layout
         syntax_layout = QGridLayout(syntax_group)
+        if sys.platform == "darwin":
+            # Default spacing is too big on Mac
+            syntax_layout.setVerticalSpacing(2 * AppStyle. MarginSize)
+
         btns = [self.schemes_combobox, edit_button, self.reset_button,
                 create_button, self.delete_button]
         for i, btn in enumerate(btns):
@@ -110,26 +132,76 @@ class AppearanceConfigPage(PluginConfigPage):
         # Fonts widgets
         self.plain_text_font = self.create_fontgroup(
             option='font',
-            title=_("Plain text"),
+            title=_("Monospace"),
             fontfilters=QFontComboBox.MonospacedFonts,
             without_group=True)
 
-        self.rich_text_font = self.create_fontgroup(
-            option='rich_font',
-            title=_("Rich text"),
+        self.app_font = self.create_fontgroup(
+            option='app_font',
+            title=_("Interface"),
+            fontfilters=QFontComboBox.ProportionalFonts,
+            restart=True,
             without_group=True)
 
-        # Fonts layouts
-        fonts_layout = QGridLayout(fonts_group)
-        fonts_layout.addWidget(self.plain_text_font.fontlabel, 0, 0)
-        fonts_layout.addWidget(self.plain_text_font.fontbox, 0, 1)
-        fonts_layout.addWidget(self.plain_text_font.sizelabel, 0, 2)
-        fonts_layout.addWidget(self.plain_text_font.sizebox, 0, 3)
-        fonts_layout.addWidget(self.rich_text_font.fontlabel, 1, 0)
-        fonts_layout.addWidget(self.rich_text_font.fontbox, 1, 1)
-        fonts_layout.addWidget(self.rich_text_font.sizelabel, 1, 2)
-        fonts_layout.addWidget(self.rich_text_font.sizebox, 1, 3)
-        fonts_layout.setRowStretch(fonts_layout.rowCount(), 1)
+        # System font checkbox
+        if sys.platform == 'darwin':
+            system_font_tip = _("Changing the interface font does not work "
+                                "reliably on macOS")
+        else:
+            system_font_tip = None
+
+        system_font_checkbox = self.create_checkbox(
+            _("Use the system default interface font"),
+            'use_system_font',
+            restart=True,
+            tip=system_font_tip
+        )
+
+        # Preview widgets
+        preview_editor_label = QLabel(_("Editor"))
+        self.preview_editor = SimpleCodeEditor(self)
+        self.preview_editor.setFixedWidth(260)
+        self.preview_editor.set_language('Python')
+        self.preview_editor.set_text(PREVIEW_TEXT)
+        self.preview_editor.set_blanks_enabled(False)
+        self.preview_editor.set_scrollpastend_enabled(False)
+
+        preview_interface_label = QLabel(_("Interface font"))
+        self.preview_interface = QLabel("Sample text")
+        self.preview_interface.setFixedWidth(260)
+        self.preview_interface.setFixedHeight(45)
+        self.preview_interface.setWordWrap(True)
+        self.preview_interface.setTextInteractionFlags(
+            Qt.TextEditorInteraction
+        )
+        self.preview_interface.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
+
+        preview_interface_label_css = qstylizer.style.StyleSheet()
+        preview_interface_label_css.QLabel.setValues(
+            border=f"1px solid {SpyderPalette.COLOR_BACKGROUND_4}",
+            borderRadius=SpyderPalette.SIZE_BORDER_RADIUS,
+            backgroundColor=SpyderPalette.COLOR_BACKGROUND_2,
+        )
+        self.preview_interface.setStyleSheet(
+            preview_interface_label_css.toString()
+        )
+
+        # Fonts layout
+        fonts_grid_layout = QGridLayout()
+        fonts_grid_layout.addWidget(self.plain_text_font.fontlabel, 0, 0)
+        fonts_grid_layout.addWidget(self.plain_text_font.fontbox, 0, 1)
+        fonts_grid_layout.addWidget(self.plain_text_font.sizebox, 0, 2)
+        fonts_grid_layout.addWidget(self.app_font.fontlabel, 2, 0)
+        fonts_grid_layout.addWidget(self.app_font.fontbox, 2, 1)
+        fonts_grid_layout.addWidget(self.app_font.sizebox, 2, 2)
+        fonts_grid_layout.setRowStretch(fonts_grid_layout.rowCount(), 1)
+
+        fonts_layout = QVBoxLayout()
+        fonts_layout.addLayout(fonts_grid_layout)
+        fonts_layout.addSpacing(5)
+        fonts_layout.addWidget(system_font_checkbox)
+
+        fonts_group.setLayout(fonts_layout)
 
         # Left options layout
         options_layout = QVBoxLayout()
@@ -137,37 +209,101 @@ class AppearanceConfigPage(PluginConfigPage):
         options_layout.addWidget(syntax_group)
         options_layout.addWidget(fonts_group)
 
-        # Right preview layout
-        preview_group = QGroupBox(_("Preview"))
+        # Right previews layout
+        preview_group = QGroupBox(_("Previews"))
         preview_layout = QVBoxLayout()
+        preview_layout.addSpacing(AppStyle.MarginSize)
+        preview_layout.addWidget(preview_editor_label)
         preview_layout.addWidget(self.preview_editor)
+        preview_layout.addSpacing(2 * AppStyle.MarginSize)
+        preview_layout.addWidget(preview_interface_label)
+        preview_layout.addWidget(self.preview_interface)
         preview_group.setLayout(preview_layout)
 
         # Combined layout
         combined_layout = QGridLayout()
-        combined_layout.setRowStretch(0, 1)
-        combined_layout.setColumnStretch(1, 100)
+        combined_layout.setHorizontalSpacing(AppStyle.MarginSize * 5)
         combined_layout.addLayout(options_layout, 0, 0)
         combined_layout.addWidget(preview_group, 0, 1)
-        self.setLayout(combined_layout)
+
+        # Final layout
+        # Note: This is necessary to prevent the layout from growing downward
+        # indefinitely.
+        final_layout = QVBoxLayout()
+        final_layout.addLayout(combined_layout)
+        final_layout.addStretch()
+        self.setLayout(final_layout)
 
         # Signals and slots
         create_button.clicked.connect(self.create_new_scheme)
         edit_button.clicked.connect(self.edit_scheme)
         self.reset_button.clicked.connect(self.reset_to_default)
         self.delete_button.clicked.connect(self.delete_scheme)
-        self.schemes_combobox.currentIndexChanged.connect(self.update_preview)
+        self.schemes_combobox.currentIndexChanged.connect(
+            lambda index: self.update_editor_preview()
+        )
+        self.schemes_combobox.sig_popup_is_hidden.connect(
+            self.update_editor_preview
+        )
+        self.schemes_combobox.sig_item_in_popup_changed.connect(
+            lambda scheme_name: self.update_editor_preview(
+                scheme_name=scheme_name
+            )
+        )
         self.schemes_combobox.currentIndexChanged.connect(self.update_buttons)
+        self.plain_text_font.fontbox.currentFontChanged.connect(
+            lambda font: self.update_editor_preview()
+        )
+        self.plain_text_font.fontbox.sig_popup_is_hidden.connect(
+            self.update_editor_preview
+        )
+        self.plain_text_font.fontbox.sig_item_in_popup_changed.connect(
+            lambda font_family: self.update_editor_preview(
+                scheme_name=None, font_family=font_family
+            )
+        )
+        self.plain_text_font.sizebox.valueChanged.connect(
+            lambda value: self.update_editor_preview()
+        )
+        self.app_font.fontbox.currentFontChanged.connect(
+            lambda font: self.update_interface_preview()
+        )
+        self.app_font.fontbox.sig_popup_is_hidden.connect(
+            self.update_interface_preview
+        )
+        self.app_font.fontbox.sig_item_in_popup_changed.connect(
+            self.update_interface_preview
+        )
+        self.app_font.sizebox.valueChanged.connect(
+            lambda value: self.update_interface_preview()
+        )
+        system_font_checkbox.checkbox.stateChanged.connect(
+            self.update_app_font_group
+        )
 
         # Setup
         for name in names:
             self.scheme_editor_dialog.add_color_scheme_stack(name)
 
+        valid_custom_names = []
         for name in custom_names:
-            self.scheme_editor_dialog.add_color_scheme_stack(name, custom=True)
+            try:
+                self.scheme_editor_dialog.add_color_scheme_stack(
+                    name, custom=True
+                )
+                valid_custom_names.append(name)
+            except configparser.NoOptionError:
+                # Ignore invalid custom syntax highlighting themes
+                # See spyder-ide/spyder#22492
+                pass
 
+        self.set_option("custom_names", valid_custom_names)
+
+        if sys.platform == 'darwin':
+            system_font_checkbox.checkbox.setEnabled(False)
+        self.update_app_font_group(system_font_checkbox.checkbox.isChecked())
         self.update_combobox()
-        self.update_preview()
+        self.update_editor_preview()
 
     def get_font(self, option):
         """Return global font used in Spyder."""
@@ -175,11 +311,14 @@ class AppearanceConfigPage(PluginConfigPage):
 
     def set_font(self, font, option):
         """Set global font used in Spyder."""
-        # Update fonts in all plugins
         set_font(font, option=option)
-        plugins = self.main.widgetlist + self.main.thirdparty_plugins
-        for plugin in plugins:
-            plugin.update_font()
+
+        # The app font can't be set in place. Instead, it requires a restart
+        if option != 'app_font':
+            # Update fonts for all plugins
+            plugins = self.main.widgetlist + self.main.thirdparty_plugins
+            for plugin in plugins:
+                plugin.update_font()
 
     def apply_settings(self):
         ui_theme = self.get_option('ui_theme')
@@ -210,11 +349,11 @@ class AppearanceConfigPage(PluginConfigPage):
             CONF.restore_notifications(section='appearance', option=option)
 
         self.update_combobox()
-        self.update_preview()
+        self.update_editor_preview()
 
         return set(self.changed_options)
 
-    # Helpers
+    # ---- Helpers
     # -------------------------------------------------------------------------
     @property
     def current_scheme_name(self):
@@ -232,6 +371,24 @@ class AppearanceConfigPage(PluginConfigPage):
     def current_ui_theme_index(self):
         return self.ui_combobox.currentIndex()
 
+    # ---- Qt methods
+    # -------------------------------------------------------------------------
+    def showEvent(self, event):
+        """Adjustments when the page is shown."""
+        super().showEvent(event)
+
+        if not self._is_shown:
+            # Set the right interface font for Mac in the respective combobox,
+            # so that preview_interface shows it appropriately.
+            if sys.platform == "darwin":
+                index = self.app_font.fontbox.findText("SF Pro")
+                if index != -1:
+                    self.app_font.fontbox.setCurrentIndex(index)
+
+        self._is_shown = True
+
+    # ---- Update contents
+    # -------------------------------------------------------------------------
     def update_combobox(self):
         """Recreates the combobox contents."""
         index = self.current_scheme_index
@@ -287,36 +444,46 @@ class AppearanceConfigPage(PluginConfigPage):
         self.delete_button.setEnabled(delete_enabled)
         self.reset_button.setEnabled(not delete_enabled)
 
-    def update_preview(self, index=None, scheme_name=None):
-        """
-        Update the color scheme of the preview editor and adds text.
-
-        Note
-        ----
-        'index' is needed, because this is triggered by a signal that sends
-        the selected index.
-        """
-        text = ('"""A string"""\n\n'
-                '# A comment\n\n'
-                'class Foo(object):\n'
-                '    def __init__(self):\n'
-                '        bar = 42\n'
-                '        print(bar)\n'
-                )
-
+    def update_editor_preview(self, scheme_name=None, font_family=None):
+        """Update the color scheme of the preview editor and adds text."""
         if scheme_name is None:
             scheme_name = self.current_scheme
+        else:
+            scheme_name = self.scheme_choices_dict[scheme_name]
 
+        if font_family is None:
+            plain_text_font = self.plain_text_font.fontbox.currentFont()
+        else:
+            plain_text_font = QFont(font_family)
+
+        plain_text_font.setPointSize(self.plain_text_font.sizebox.value())
         self.preview_editor.setup_editor(
-            font=get_font(),
-            color_scheme=scheme_name,
-            show_blanks=False,
-            scroll_past_end=False,
+            font=plain_text_font,
+            color_scheme=scheme_name
         )
-        self.preview_editor.set_language('Python')
-        self.preview_editor.set_text(text)
 
-    # Actions
+    def update_interface_preview(self, font_family=None):
+        """Update the interface preview label."""
+        if font_family is None:
+            app_font = self.app_font.fontbox.currentFont()
+        else:
+            app_font = QFont(font_family)
+
+        app_font.setPointSize(self.app_font.sizebox.value())
+        self.preview_interface.setFont(app_font)
+
+    def update_app_font_group(self, state):
+        """Update app font group enabled state."""
+        subwidgets = ['fontlabel', 'fontbox', 'sizebox']
+
+        if state:
+            for widget in subwidgets:
+                getattr(self.app_font, widget).setEnabled(False)
+        else:
+            for widget in subwidgets:
+                getattr(self.app_font, widget).setEnabled(True)
+
+    # ---- Actions
     # -------------------------------------------------------------------------
     def create_new_scheme(self):
         """Creates a new color scheme with a custom name."""
@@ -375,7 +542,11 @@ class AppearanceConfigPage(PluginConfigPage):
                 option = "temp/{0}".format(key)
                 value = temporal_color_scheme[key]
                 self.set_option(option, value)
-            self.update_preview(scheme_name='temp')
+
+            if not self.scheme_choices_dict.get("temp"):
+                self.scheme_choices_dict["temp"] = "temp"
+
+            self.update_editor_preview(scheme_name='temp')
 
     def delete_scheme(self):
         """Deletes the currently selected custom color scheme."""
@@ -408,7 +579,7 @@ class AppearanceConfigPage(PluginConfigPage):
                                "{0}/name".format(scheme_name))
 
             self.update_combobox()
-            self.update_preview()
+            self.update_editor_preview()
 
     def set_scheme(self, scheme_name):
         """
@@ -439,7 +610,7 @@ class AppearanceConfigPage(PluginConfigPage):
         We need to do this because when applying settings we can't
         detect correctly the current theme.
         """
-        return dark_color(QStylePalette.COLOR_BACKGROUND_1)
+        return dark_color(SpyderPalette.COLOR_BACKGROUND_1)
 
     def color_scheme_and_ui_theme_mismatch(self, color_scheme, ui_theme):
         """
