@@ -340,7 +340,7 @@ def test_editor_docstring_below_def_by_shortcut(qtbot, editor_auto_docstring,
 
     cursor = editor.textCursor()
     cursor.movePosition(QTextCursor.NextBlock)
-    cursor.setPosition(QTextCursor.End, QTextCursor.MoveAnchor)
+    cursor.movePosition(QTextCursor.End, QTextCursor.MoveAnchor)
     editor.setTextCursor(cursor)
 
     editor.writer_docstring.write_docstring_for_shortcut()
@@ -376,7 +376,7 @@ def test_editor_docstring_delayed_popup(qtbot, editor_auto_docstring,
 
     cursor = editor.textCursor()
     cursor.movePosition(QTextCursor.NextBlock)
-    cursor.setPosition(QTextCursor.EndOfLine, QTextCursor.MoveAnchor)
+    cursor.movePosition(QTextCursor.EndOfLine, QTextCursor.MoveAnchor)
     editor.setTextCursor(cursor)
 
     qtbot.keyPress(editor, Qt.Key_Space)
@@ -618,7 +618,7 @@ def test_docstring_annotated_call(editor_auto_docstring, text, expected):
 
     cursor = editor.textCursor()
     cursor.movePosition(QTextCursor.NextBlock)
-    cursor.setPosition(QTextCursor.End, QTextCursor.MoveAnchor)
+    cursor.movePosition(QTextCursor.End, QTextCursor.MoveAnchor)
     editor.setTextCursor(cursor)
 
     editor.writer_docstring.write_docstring_for_shortcut()
@@ -657,6 +657,63 @@ def test_docstring_line_break(editor_auto_docstring, text, expected):
     editor = editor_auto_docstring
     editor.set_text(text)
 
+    cursor = editor.textCursor()
+    cursor.movePosition(QTextCursor.NextBlock)
+    cursor.movePosition(QTextCursor.End, QTextCursor.MoveAnchor)
+    editor.setTextCursor(cursor)
+
+    editor.writer_docstring.write_docstring_for_shortcut()
+
+    assert editor.toPlainText() == expected
+
+
+@pytest.mark.parametrize(
+    'text, expected',
+    [
+        ('''  def test(v: str = "#"):  # comment, with '#' and "#"
+      ''',
+         '''  def test(v: str = "#"):  # comment, with '#' and "#"
+      """\n      \n
+      Parameters
+      ----------
+      v : str, optional
+          DESCRIPTION. The default is "#".
+
+      Returns
+      -------
+      None.
+
+      """
+      '''),
+        ('''  def test(v1: str = "#", # comment, with '#' and "#"
+         v2: str = '#') -> str:
+      ''',
+         '''  def test(v1: str = "#", # comment, with '#' and "#"
+         v2: str = '#') -> str:
+      """\n      \n
+      Parameters
+      ----------
+      v1 : str, optional
+          DESCRIPTION. The default is "#".
+      v2 : str, optional
+          DESCRIPTION. The default is '#'.
+
+      Returns
+      -------
+      str
+          DESCRIPTION.
+
+      """
+      '''),
+    ])
+def test_docstring_comments(editor_auto_docstring, text, expected):
+    """
+    Test auto docstring with comments on lines of function definition.
+    """
+    CONF.set('editor', 'docstring_type', 'Numpydoc')
+    editor = editor_auto_docstring
+    editor.set_text(text)
+
     cursor = editor.textCursor()
     cursor.movePosition(QTextCursor.NextBlock)
     cursor.setPosition(QTextCursor.End, QTextCursor.MoveAnchor)
