@@ -199,7 +199,10 @@ class AutosaveForPlugin(object):
         the pid files.
         """
         files_to_recover, pidfiles = self.get_files_to_recover()
-        parent = self.editor if running_under_pytest() else self.editor.main
+        parent = (
+            self.editor if running_under_pytest() else
+            self.editor._get_mainwindow()
+        )
         dialog = RecoveryDialog(files_to_recover, parent=parent)
         dialog.exec_if_nonempty()
         self.recover_files_to_open = dialog.files_to_open[:]
@@ -434,7 +437,7 @@ class AutosaveForStack(object):
             old_hash = self.file_hashes[old_name]
         except KeyError:
             # This should not happen, but it does: spyder-ide/spyder#12396
-            logger.error('KeyError when handling rename %s -> %s',
+            logger.debug('KeyError when handling rename %s -> %s',
                          old_name, new_name)
             old_hash = None
         self.remove_autosave_file(old_name)
