@@ -6,16 +6,19 @@
 
 # Standard library imports
 import fnmatch
+import math
 import os.path as osp
 import re
 
 # Third party imports
+from qtpy import PYQT5, PYQT6
 from qtpy.QtCore import Signal
-from qtpy.QtWidgets import QHBoxLayout, QInputDialog, QLabel
+from qtpy.QtGui import QFontMetricsF
+from qtpy.QtWidgets import QInputDialog, QLabel, QStackedWidget, QVBoxLayout
 
 # Local imports
 from spyder.api.config.decorators import on_conf_change
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.api.widgets.main_widget import PluginMainWidget
 from spyder.plugins.findinfiles.widgets.results_browser import (
     ON, ResultsBrowser)
@@ -23,17 +26,17 @@ from spyder.plugins.findinfiles.widgets.combobox import (
     MAX_PATH_HISTORY, SearchInComboBox)
 from spyder.plugins.findinfiles.widgets.search_thread import SearchThread
 from spyder.utils.misc import regexp_error_msg
-from spyder.utils.palette import QStylePalette, SpyderPalette
+from spyder.utils.palette import SpyderPalette
+from spyder.utils.stylesheet import AppStyle
 from spyder.widgets.comboboxes import PatternComboBox
-
-
-# Localization
-_ = get_translation('spyder')
+from spyder.widgets.helperwidgets import PaneEmptyWidget
 
 
 # ---- Constants
 # -----------------------------------------------------------------------------
-MAIN_TEXT_COLOR = QStylePalette.COLOR_TEXT_1
+MAIN_TEXT_COLOR = SpyderPalette.COLOR_TEXT_1
+MAX_COMBOBOX_WIDTH = AppStyle.FindMinWidth + 80  # In pixels
+MIN_COMBOBOX_WIDTH = AppStyle.FindMinWidth - 80  # In pixels
 
 
 # ---- Enums
@@ -75,6 +78,7 @@ class FindInFilesWidgetToolbarItems:
     ExcludePatternCombo = 'exclude_pattern_combo'
     Stretcher1 = 'stretcher_1'
     SearchInCombo = 'search_in_combo'
+    Stretcher2 = 'stretcher_2'
 
 
 # ---- Main widget
@@ -84,7 +88,11 @@ class FindInFilesWidget(PluginMainWidget):
     Find in files main widget.
     """
 
+    # PluginMainWidget constants
     ENABLE_SPINNER = True
+    MARGIN_TOP = AppStyle.MarginSize + 5
+
+    # Other constants
     REGEX_INVALID = f"background-color:{SpyderPalette.COLOR_ERROR_2};"
     REGEX_ERROR = _("Regular expression error")
 
@@ -120,7 +128,10 @@ class FindInFilesWidget(PluginMainWidget):
     """
 
     def __init__(self, name=None, plugin=None, parent=None):
-        super().__init__(name, plugin, parent=parent)
+        if PYQT5 or PYQT6:
+            super().__init__(name, plugin, parent=parent)
+        else:
+            PluginMainWidget.__init__(self, name, plugin, parent=parent)
         self.set_conf('text_color', MAIN_TEXT_COLOR)
         self.set_conf('hist_limit', MAX_PATH_HISTORY)
 
@@ -131,6 +142,10 @@ class FindInFilesWidget(PluginMainWidget):
         self.running = False
         self.more_options_action = None
         self.extras_toolbar = None
+        self._search_in_label_width = None
+        self._exclude_label_width = None
+        self._is_shown = False
+        self._is_first_time = False
 
         search_text = self.get_conf('search_text', '')
         path_history = self.get_conf('path_history', [])
@@ -146,12 +161,20 @@ class FindInFilesWidget(PluginMainWidget):
             path_history = [path_history]
 
         # Widgets
-        self.search_text_edit = PatternComboBox(
+        self.pane_empty = PaneEmptyWidget(
             self,
-            search_text,
-            id_=FindInFilesWidgetToolbarItems.SearchPatternCombo
+            "find_empty",
+            _("Nothing searched for yet"),
+            _("Search the content of text files in any directory using the "
+              "search box.")
         )
 
+        self.search_text_edit = PatternComboBox(
+            self,
+            items=search_text,
+            adjust_to_minimum=False,
+            id_=FindInFilesWidgetToolbarItems.SearchPatternCombo
+        )
         self.search_text_edit.lineEdit().setPlaceholderText(
             _('Write text to search'))
 
@@ -162,8 +185,14 @@ class FindInFilesWidget(PluginMainWidget):
         self.exclude_label.ID = FindInFilesWidgetToolbarItems.ExcludeLabel
 
         self.path_selection_combo = SearchInComboBox(
-            path_history, self,
-            id_=FindInFilesWidgetToolbarItems.SearchInCombo)
+            path_history,
+            self,
+            id_=FindInFilesWidgetToolbarItems.SearchInCombo
+        )
+        self.path_selection_combo.setMinimumSize(
+            MIN_COMBOBOX_WIDTH, AppStyle.FindHeight
+        )
+        self.path_selection_combo.setMaximumWidth(MAX_COMBOBOX_WIDTH)
 
         self.exclude_pattern_edit = PatternComboBox(
             self,
@@ -171,6 +200,10 @@ class FindInFilesWidget(PluginMainWidget):
             _("Exclude pattern"),
             id_=FindInFilesWidgetToolbarItems.ExcludePatternCombo
         )
+        self.exclude_pattern_edit.setMinimumSize(
+            MIN_COMBOBOX_WIDTH, AppStyle.FindHeight
+        )
+        self.exclude_pattern_edit.setMaximumWidth(MAX_COMBOBOX_WIDTH)
 
         self.result_browser = ResultsBrowser(
             self,
@@ -179,7 +212,6 @@ class FindInFilesWidget(PluginMainWidget):
         )
 
         # Setup
-        self.exclude_label.setBuddy(self.exclude_pattern_edit)
         exclude_idx = self.get_conf('exclude_index', None)
         if (exclude_idx is not None and exclude_idx >= 0
                 and exclude_idx < self.exclude_pattern_edit.count()):
@@ -190,8 +222,12 @@ class FindInFilesWidget(PluginMainWidget):
             search_in_index)
 
         # Layout
-        layout = QHBoxLayout()
-        layout.addWidget(self.result_browser)
+        self.stacked_widget = QStackedWidget(self)
+        self.stacked_widget.addWidget(self.result_browser)
+        self.stacked_widget.addWidget(self.pane_empty)
+
+        layout = QVBoxLayout()
+        layout.addWidget(self.stacked_widget)
         self.setLayout(layout)
 
         # Signals
@@ -205,9 +241,8 @@ class FindInFilesWidget(PluginMainWidget):
             self.sig_max_results_reached)
         self.result_browser.sig_max_results_reached.connect(
             self._stop_and_reset_thread)
-        self.search_text_edit.sig_resized.connect(self._update_size)
 
-    # --- PluginMainWidget API
+    # ---- PluginMainWidget API
     # ------------------------------------------------------------------------
     def get_title(self):
         return _("Find")
@@ -272,6 +307,7 @@ class FindInFilesWidget(PluginMainWidget):
         self.set_max_results_action = self.create_action(
             FindInFilesWidgetActions.MaxResults,
             text=_('Set maximum number of results'),
+            icon=self.create_icon("transparent"),
             tip=_('Set maximum number of results'),
             triggered=lambda x=None: self.set_max_results(),
         )
@@ -291,10 +327,10 @@ class FindInFilesWidget(PluginMainWidget):
         self.extras_toolbar = self.create_toolbar(
             FindInFilesWidgetToolbars.Exclude)
 
-        stretcher = self.create_stretcher()
-        stretcher.ID = FindInFilesWidgetToolbarItems.Stretcher1
+        stretcher1 = self.create_stretcher(
+            FindInFilesWidgetToolbarItems.Stretcher1)
         for item in [self.exclude_label, self.exclude_pattern_edit,
-                     self.exclude_regexp_action, stretcher]:
+                     self.exclude_regexp_action, stretcher1]:
             self.add_item_to_toolbar(
                 item,
                 toolbar=self.extras_toolbar,
@@ -304,7 +340,10 @@ class FindInFilesWidget(PluginMainWidget):
         # Location toolbar
         location_toolbar = self.create_toolbar(
             FindInFilesWidgetToolbars.Location)
-        for item in [self.search_in_label, self.path_selection_combo]:
+        stretcher2 = self.create_stretcher(
+            FindInFilesWidgetToolbarItems.Stretcher2)
+        for item in [self.search_in_label, self.path_selection_combo,
+                     stretcher2]:
             self.add_item_to_toolbar(
                 item,
                 toolbar=location_toolbar,
@@ -317,9 +356,13 @@ class FindInFilesWidget(PluginMainWidget):
             menu=menu,
         )
 
+        # Set pane_empty widget at the beginning
+        self.stacked_widget.setCurrentWidget(self.pane_empty)
+
     def update_actions(self):
         self.find_action.setIcon(self.create_icon(
-            'stop' if self.running else 'find'))
+            'stop' if self.running else 'find')
+        )
 
         if self.extras_toolbar and self.more_options_action:
             self.extras_toolbar.setVisible(
@@ -327,9 +370,6 @@ class FindInFilesWidget(PluginMainWidget):
 
     @on_conf_change(option='more_options')
     def on_more_options_update(self, value):
-        self.exclude_pattern_edit.setMinimumWidth(
-            self.search_text_edit.width())
-
         if value:
             icon = self.create_icon('options_less')
             tip = _('Hide advanced options')
@@ -340,6 +380,34 @@ class FindInFilesWidget(PluginMainWidget):
         if self.extras_toolbar:
             self.extras_toolbar.setVisible(value)
 
+        # These adjustments can only be done when the widget is visible.
+        if self._is_shown:
+            if value:
+                # Resize either the search_in or exclude label so that their
+                # comboboxes are aligned to the left.
+                if self._search_in_label_width > self._exclude_label_width:
+                    self.exclude_label.setMinimumSize(
+                        self._exclude_label_width +
+                        (self._search_in_label_width -
+                         self._exclude_label_width),
+                        AppStyle.FindHeight
+                    )
+                else:
+                    self.search_in_label.setMinimumSize(
+                        self._search_in_label_width +
+                        (self._exclude_label_width -
+                         self._search_in_label_width),
+                        AppStyle.FindHeight
+                    )
+            else:
+                # Restore initial search_in label width when it's shorter than
+                # exclude_label to not show an empty space next to it.
+                if self._search_in_label_width < self._exclude_label_width:
+                    self.search_in_label.setMinimumSize(
+                        self._search_in_label_width,
+                        AppStyle.FindHeight
+                    )
+
         if self.more_options_action:
             self.more_options_action.setIcon(icon)
             self.more_options_action.setToolTip(tip)
@@ -348,11 +416,41 @@ class FindInFilesWidget(PluginMainWidget):
     def on_max_results_update(self, value):
         self.result_browser.set_max_results(value)
 
-    # --- Private API
+    # ---- Qt methods
     # ------------------------------------------------------------------------
-    def _update_size(self, size, old_size):
-        self.exclude_pattern_edit.setMinimumWidth(size.width())
+    def showEvent(self, event):
+        """Adjustments when the widget is shown."""
+        if not self._is_shown:
+            # Save default widths of search_in and exclude labels
+            self._search_in_label_width = self.search_in_label.size().width()
 
+            if not self.extras_toolbar.isVisible():
+                # This correctly computes the exclude label width when the
+                # extras_toolbar is not visible.
+                metrics = QFontMetricsF(self.font())
+                exclude_text_width = metrics.width(self.exclude_label.text())
+                self._exclude_label_width = (
+                    math.ceil(exclude_text_width) + self.font().pointSize()
+                )
+            else:
+                self._exclude_label_width = self.exclude_label.size().width()
+
+            self._is_shown = True
+
+        super().showEvent(event)
+
+    def resizeEvent(self, event):
+        """Adjustments when the widget is resized."""
+        super().resizeEvent(event)
+
+        # This recomputes the result items width according to this widget's
+        # width, which makes the UI be rendered as expected.
+        # NOTE: Don't debounce or throttle `set_width` because then it wouldn't
+        # do its job as expected.
+        self.result_browser.set_width()
+
+    # ---- Private API
+    # ------------------------------------------------------------------------
     def _get_options(self):
         """
         Get search options.
@@ -466,7 +564,7 @@ class FindInFilesWidget(PluginMainWidget):
             if self.search_thread.isRunning():
                 if ignore_results:
                     self.search_thread.sig_finished.disconnect(
-                        self.search_complete)
+                        self._handle_search_complete)
                 self.search_thread.stop()
                 self.search_thread.wait()
 
@@ -477,7 +575,7 @@ class FindInFilesWidget(PluginMainWidget):
         self.stop_spinner()
         self.update_actions()
 
-    # --- Public API
+    # ---- Public API
     # ------------------------------------------------------------------------
     @property
     def path(self):
@@ -560,6 +658,12 @@ class FindInFilesWidget(PluginMainWidget):
         If there is no search running, this will start the search. If there is
         a search running, this will stop it.
         """
+        # Show result_browser the first time a user performs a search and leave
+        # it shown afterwards.
+        if not self._is_first_time:
+            self.stacked_widget.setCurrentWidget(self.result_browser)
+            self._is_first_time = True
+
         if self.running:
             self.stop()
         else:
