@@ -23,29 +23,28 @@ import time
 from itertools import islice
 
 # Third party imports
-from qtpy import PYQT5
+from qtpy import PYQT5, PYQT6
 from qtpy.compat import getopenfilename, getsavefilename
 from qtpy.QtCore import QByteArray, QProcess, QProcessEnvironment, Qt, Signal
 from qtpy.QtGui import QColor
 from qtpy.QtWidgets import (QApplication, QLabel, QMessageBox, QTreeWidget,
-                            QTreeWidgetItem, QVBoxLayout)
+                            QTreeWidgetItem, QStackedWidget, QVBoxLayout)
 
 # Local imports
 from spyder.api.config.decorators import on_conf_change
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.api.widgets.main_widget import PluginMainWidget
 from spyder.api.widgets.mixins import SpyderWidgetMixin
-from spyder.config.base import get_conf_path, running_in_mac_app
+from spyder.config.base import get_conf_path
 from spyder.plugins.variableexplorer.widgets.texteditor import TextEditor
 from spyder.py3compat import to_text_string
 from spyder.utils.misc import get_python_executable, getcwd_or_home
-from spyder.utils.palette import SpyderPalette, QStylePalette
+from spyder.utils.palette import SpyderPalette
 from spyder.utils.programs import shell_split
 from spyder.utils.qthelpers import get_item_user_text, set_item_user_text
 from spyder.widgets.comboboxes import PythonModulesComboBox
+from spyder.widgets.helperwidgets import PaneEmptyWidget
 
-# Localization
-_ = get_translation('spyder')
 
 # Logging
 logger = logging.getLogger(__name__)
@@ -53,7 +52,7 @@ logger = logging.getLogger(__name__)
 
 # --- Constants
 # ----------------------------------------------------------------------------
-MAIN_TEXT_COLOR = QStylePalette.COLOR_TEXT_1
+MAIN_TEXT_COLOR = SpyderPalette.COLOR_TEXT_1
 
 
 class ProfilerWidgetActions:
@@ -175,12 +174,24 @@ class ProfilerWidget(PluginMainWidget):
         self.filecombo = PythonModulesComboBox(
             self, id_=ProfilerWidgetMainToolbarItems.FileCombo)
         self.datatree = ProfilerDataTree(self)
+        self.pane_empty = PaneEmptyWidget(
+            self,
+            "code-profiler",
+            _("Code not profiled yet"),
+            _("Profile your code to explore which functions and methods "
+              "took the longest to run and were called the most, "
+              "and find out where to optimize it.")
+        )
         self.datelabel = QLabel()
         self.datelabel.ID = ProfilerWidgetInformationToolbarItems.DateLabel
 
         # Layout
+        self.stacked_widget = QStackedWidget(self)
+        self.stacked_widget.addWidget(self.pane_empty)
+        self.stacked_widget.addWidget(self.datatree)
+
         layout = QVBoxLayout()
-        layout.addWidget(self.datatree)
+        layout.addWidget(self.stacked_widget)
         self.setLayout(layout)
 
         # Signals
@@ -507,7 +518,7 @@ class ProfilerWidget(PluginMainWidget):
         if wdir is None:
             wdir = self._last_wdir
             if wdir is None:
-                wdir = osp.basename(filename)
+                wdir = osp.dirname(filename)
 
         if args is None:
             args = self._last_args
@@ -535,6 +546,7 @@ class ProfilerWidget(PluginMainWidget):
             proc_env.insert(k, v)
         proc_env.insert("PYTHONIOENCODING", "utf8")
         proc_env.remove('PYTHONPATH')
+        proc_env.remove('PYTHONEXECUTABLE') # needed on macOS to set sys.path correctly
         if self.pythonpath is not None:
             logger.debug(f"Pass Pythonpath {self.pythonpath} to process")
             proc_env.insert('PYTHONPATH', os.pathsep.join(self.pythonpath))
@@ -542,11 +554,6 @@ class ProfilerWidget(PluginMainWidget):
 
         executable = self.get_conf('executable', section='main_interpreter')
 
-        if not running_in_mac_app(executable):
-            env = self.process.processEnvironment()
-            env.remove('PYTHONHOME')
-            self.process.setProcessEnvironment(env)
-
         self.output = ''
         self.error_output = ''
         self.running = True
@@ -611,6 +618,7 @@ class ProfilerWidget(PluginMainWidget):
         self.datelabel.setText(_('Sorting data, please wait...'))
         QApplication.processEvents()
 
+        self.stacked_widget.setCurrentWidget(self.datatree)
         self.datatree.load_data(self.DATAPATH)
         self.datatree.show_tree()
 
@@ -664,7 +672,7 @@ class ProfilerDataTree(QTreeWidget, SpyderWidgetMixin):
     sig_edit_goto_requested = Signal(str, int, str)
 
     def __init__(self, parent=None):
-        if PYQT5:
+        if PYQT5 or PYQT6:
             super().__init__(parent, class_parent=parent)
         else:
             QTreeWidget.__init__(self, parent)
