@@ -13,21 +13,27 @@ import os.path as osp
 import sys
 
 # Third party imports
-from qtpy import PYQT5
+from qtpy import PYQT5, PYQT6
 from qtpy.compat import getexistingdirectory
-from qtpy.QtCore import Qt, Signal, Slot
+from qtpy.QtCore import QSize, Qt, Signal, Slot
+from qtpy.QtGui import QFontMetrics
 from qtpy.QtWidgets import (QDialog, QDialogButtonBox, QHBoxLayout,
                             QListWidget, QListWidgetItem, QMessageBox,
                             QVBoxLayout, QLabel)
 
 # Local imports
+from spyder.api.widgets.dialogs import SpyderDialogButtonBox
 from spyder.api.widgets.mixins import SpyderWidgetMixin
 from spyder.config.base import _
 from spyder.plugins.pythonpath.utils import check_path, get_system_pythonpath
 from spyder.utils.environ import get_user_env, set_user_env
-from spyder.utils.icon_manager import ima
 from spyder.utils.misc import getcwd_or_home
-from spyder.utils.stylesheet import PANES_TOOLBAR_STYLESHEET
+from spyder.utils.stylesheet import (
+    AppStyle,
+    MAC,
+    PANES_TOOLBAR_STYLESHEET,
+    WIN
+)
 
 
 class PathManagerToolbuttons:
@@ -52,7 +58,7 @@ class PathManager(QDialog, SpyderWidgetMixin):
     def __init__(self, parent, path=None, project_path=None,
                  not_active_path=None, sync=True):
         """Path manager dialog."""
-        if PYQT5:
+        if PYQT5 or PYQT6:
             super().__init__(parent, class_parent=parent)
         else:
             QDialog.__init__(self, parent)
@@ -60,12 +66,14 @@ class PathManager(QDialog, SpyderWidgetMixin):
 
         assert isinstance(path, (tuple, type(None)))
 
-        # Match buttons style with the rest of Spyder
-        self.setStyleSheet(str(PANES_TOOLBAR_STYLESHEET))
+        # Style
+        # NOTE: This needs to be here so all buttons are styled correctly
+        self.setStyleSheet(self._stylesheet)
 
         self.path = path or ()
         self.project_path = project_path or ()
         self.not_active_path = not_active_path or ()
+
         self.last_path = getcwd_or_home()
         self.original_path_dict = None
         self.system_path = ()
@@ -90,21 +98,22 @@ class PathManager(QDialog, SpyderWidgetMixin):
         self.selection_widgets = []
         self.right_buttons = self._setup_right_toolbar()
         self.listwidget = QListWidget(self)
-        self.bbox = QDialogButtonBox(QDialogButtonBox.Ok
-                                     | QDialogButtonBox.Cancel)
+        self.bbox = SpyderDialogButtonBox(
+            QDialogButtonBox.Ok | QDialogButtonBox.Cancel
+        )
         self.button_ok = self.bbox.button(QDialogButtonBox.Ok)
 
         # Widget setup
         self.setWindowTitle(_("PYTHONPATH manager"))
-        self.setWindowIcon(ima.icon('pythonpath'))
+        self.setWindowIcon(self.create_icon('pythonpath'))
         self.resize(500, 400)
         self.export_button.setVisible(os.name == 'nt' and sync)
 
         # Description
         description = QLabel(
-            _("The paths listed below will be passed to IPython consoles and "
-              "the Python language server as additional locations to search "
-              "for Python modules.")
+            _("The paths listed below will be passed to the IPython console "
+              "and to the Editor as additional locations to search for Python "
+              "modules.")
         )
         description.setWordWrap(True)
 
@@ -115,14 +124,16 @@ class PathManager(QDialog, SpyderWidgetMixin):
 
         # Middle layout
         middle_layout = QHBoxLayout()
+        middle_layout.setContentsMargins(4 if WIN else 5, 0, 0, 0)
         middle_layout.addWidget(self.listwidget)
         middle_layout.addLayout(buttons_layout)
 
         # Widget layout
         layout = QVBoxLayout()
         layout.addWidget(description)
-        layout.addSpacing(12)
+        layout.addSpacing(2 * AppStyle.MarginSize)
         layout.addLayout(middle_layout)
+        layout.addSpacing((-1 if MAC else 2) * AppStyle.MarginSize)
         layout.addWidget(self.bbox)
         self.setLayout(layout)
 
@@ -135,6 +146,8 @@ class PathManager(QDialog, SpyderWidgetMixin):
         # Setup
         self.setup()
 
+    # ---- Private methods
+    # -------------------------------------------------------------------------
     def _add_buttons_to_layout(self, widgets, layout):
         """Helper to add buttons to its layout."""
         for widget in widgets:
@@ -188,7 +201,6 @@ class PathManager(QDialog, SpyderWidgetMixin):
     def _create_item(self, path):
         """Helper to create a new list item."""
         item = QListWidgetItem(path)
-        item.setIcon(ima.icon('DirClosedIcon'))
 
         if path in self.project_path:
             item.setFlags(Qt.NoItemFlags | Qt.ItemIsUserCheckable)
@@ -204,19 +216,51 @@ class PathManager(QDialog, SpyderWidgetMixin):
 
     def _create_header(self, text):
         """Create a header for a given path section."""
-        header = QListWidgetItem(text)
+        header_item = QListWidgetItem()
+        header_widget = QLabel(text)
 
-        # Header is centered and it can't be selected
-        header.setTextAlignment(Qt.AlignHCenter)
-        header.setFlags(Qt.ItemIsEnabled)
+        # Disable item so we can remove its background color
+        header_item.setFlags(header_item.flags() & ~Qt.ItemIsEnabled)
+
+        # Header is centered
+        header_widget.setAlignment(Qt.AlignHCenter)
 
         # Make header appear in bold
-        font = header.font()
+        font = header_widget.font()
         font.setBold(True)
-        header.setFont(font)
+        header_widget.setFont(font)
 
-        return header
+        # Increase height to make header stand over paths
+        fm = QFontMetrics(font)
+        header_item.setSizeHint(
+            QSize(20, fm.capHeight() + 6 * AppStyle.MarginSize)
+        )
 
+        return header_item, header_widget
+
+    @property
+    def _stylesheet(self):
+        """Style for the list of paths"""
+        # This is necessary to match the buttons style with the rest of Spyder
+        toolbar_stylesheet = PANES_TOOLBAR_STYLESHEET.get_copy()
+        css = toolbar_stylesheet.get_stylesheet()
+
+        css.QListView.setValues(
+            padding=f"{AppStyle.MarginSize + 1}px"
+        )
+
+        css["QListView::item"].setValues(
+            padding=f"{AppStyle.MarginSize + (1 if WIN else 0)}px"
+        )
+
+        css["QListView::item:disabled"].setValues(
+            backgroundColor="transparent"
+        )
+
+        return css.toString()
+
+    # ---- Public methods
+    # -------------------------------------------------------------------------
     @property
     def editable_bottom_row(self):
         """Maximum bottom row count that is editable."""
@@ -251,9 +295,12 @@ class PathManager(QDialog, SpyderWidgetMixin):
 
         # Project path
         if self.project_path:
-            self.project_header = self._create_header(_("Project path"))
+            self.project_header, project_widget = (
+                self._create_header(_("Project path"))
+            )
             self.headers.append(self.project_header)
             self.listwidget.addItem(self.project_header)
+            self.listwidget.setItemWidget(self.project_header, project_widget)
 
             for path in self.project_path:
                 item = self._create_item(path)
@@ -261,9 +308,12 @@ class PathManager(QDialog, SpyderWidgetMixin):
 
         # Paths added by the user
         if self.user_path:
-            self.user_header = self._create_header(_("User paths"))
+            self.user_header, user_widget = (
+                self._create_header(_("User paths"))
+            )
             self.headers.append(self.user_header)
             self.listwidget.addItem(self.user_header)
+            self.listwidget.setItemWidget(self.user_header, user_widget)
 
             for path in self.user_path:
                 item = self._create_item(path)
@@ -271,9 +321,12 @@ class PathManager(QDialog, SpyderWidgetMixin):
 
         # System path
         if self.system_path:
-            self.system_header = self._create_header(_("System PYTHONPATH"))
+            self.system_header, system_widget = (
+                self._create_header(_("System PYTHONPATH"))
+            )
             self.headers.append(self.system_header)
             self.listwidget.addItem(self.system_header)
+            self.listwidget.setItemWidget(self.system_header, system_widget)
 
             for path in self.system_path:
                 item = self._create_item(path)
@@ -347,6 +400,7 @@ class PathManager(QDialog, SpyderWidgetMixin):
                 if path in self.project_path and not project_path:
                     continue
                 odict[path] = item.checkState() == Qt.Checked
+
         return odict
 
     def get_user_path(self):
@@ -358,6 +412,7 @@ class PathManager(QDialog, SpyderWidgetMixin):
             if item not in self.headers:
                 if path not in (self.project_path + self.system_path):
                     user_path.append(path)
+
         return user_path
 
     def update_paths(self, path=None, not_active_path=None, system_path=None):
@@ -415,7 +470,8 @@ class PathManager(QDialog, SpyderWidgetMixin):
 
         # Ok button only enabled if actual changes occur
         self.button_ok.setEnabled(
-            self.original_path_dict != self.get_path_dict())
+            self.original_path_dict != self.get_path_dict()
+        )
 
     @Slot()
     def add_path(self, directory=None):
@@ -453,17 +509,19 @@ class PathManager(QDialog, SpyderWidgetMixin):
         else:
             if check_path(directory):
                 if not self.user_header:
-                    self.user_header = self._create_header(_("User paths"))
+                    self.user_header, user_widget = (
+                        self._create_header(_("User paths"))
+                    )
                     self.headers.append(self.user_header)
 
-                # Add header if not visible
-                if self.listwidget.row(self.user_header) < 0:
                     if self.editable_top_row > 0:
                         header_row = self.editable_top_row - 1
                     else:
                         header_row = 0
-                    self.listwidget.insertItem(header_row,
-                                               self.user_header)
+                    self.listwidget.insertItem(header_row, self.user_header)
+                    self.listwidget.setItemWidget(
+                        self.user_header, user_widget
+                    )
 
                 # Add new path
                 item = self._create_item(directory)
@@ -517,7 +575,10 @@ class PathManager(QDialog, SpyderWidgetMixin):
                 # Remove user header if there are no more user paths
                 if len(self.user_path) == 0:
                     self.listwidget.takeItem(
-                        self.listwidget.row(self.user_header))
+                        self.listwidget.row(self.user_header)
+                    )
+                    self.headers.remove(self.user_header)
+                    self.user_header = None
 
                 # Refresh widget
                 self.refresh()
