@@ -16,33 +16,45 @@ from math import ceil
 import sys
 
 # Third party imports
+import qstylizer.style
 from qtpy.QtCore import (QEasingCurve, QPoint, QPropertyAnimation, QRectF, Qt,
                          Signal)
 from qtpy.QtGui import (QBrush, QColor, QIcon, QPainter, QPainterPath, QPen,
                         QPixmap, QRegion)
-from qtpy.QtWidgets import (QAction, QApplication, QComboBox, QDialog,
-                            QGraphicsOpacityEffect, QHBoxLayout, QLabel,
-                            QLayout, QMainWindow, QMenu, QMessageBox,
-                            QPushButton, QSpacerItem, QToolButton, QVBoxLayout,
-                            QWidget)
+from qtpy.QtWidgets import (
+    QAction,
+    QApplication,
+    QDialog,
+    QDialogButtonBox,
+    QGraphicsOpacityEffect,
+    QHBoxLayout,
+    QLabel,
+    QLayout,
+    QMainWindow,
+    QMessageBox,
+    QPushButton,
+    QSpacerItem,
+    QToolButton,
+    QVBoxLayout,
+    QWidget,
+)
 
 # Local imports
-from spyder import __docs_url__
-from spyder.api.panel import Panel
-from spyder.api.translations import get_translation
-from spyder.config.base import _
+from spyder.api.widgets.dialogs import SpyderDialogButtonBox
+from spyder.api.widgets.menus import SpyderMenu
+from spyder.api.widgets.mixins import SvgToScaledPixmap
+from spyder.api.translations import _
+from spyder.api.widgets.comboboxes import SpyderComboBox
 from spyder.plugins.layout.layouts import DefaultLayouts
 from spyder.py3compat import to_binary_string
 from spyder.utils.icon_manager import ima
 from spyder.utils.image_path_manager import get_image_path
-from spyder.utils.palette import QStylePalette, SpyderPalette
+from spyder.utils.palette import SpyderPalette
 from spyder.utils.qthelpers import add_actions, create_action
-from spyder.utils.stylesheet import DialogStyle
+from spyder.utils.stylesheet import AppStyle, DialogStyle
 
-MAIN_TOP_COLOR = MAIN_BG_COLOR = QColor(QStylePalette.COLOR_BACKGROUND_1)
 
-# Localization
-_ = get_translation('spyder')
+MAIN_TOP_COLOR = MAIN_BG_COLOR = QColor(SpyderPalette.COLOR_BACKGROUND_1)
 
 MAC = sys.platform == 'darwin'
 
@@ -337,7 +349,7 @@ class FadingTipBox(FadingDialog):
         self.label_image = QLabel()
 
         self.label_title = QLabel()
-        self.combo_title = QComboBox()
+        self.combo_title = SpyderComboBox()
         self.label_current = QLabel()
         self.label_content = QLabel()
 
@@ -379,6 +391,7 @@ class FadingTipBox(FadingDialog):
                              image: url({});
                              }}
                              '''.format(self.combobox_background.name(), arrow)
+
         # Windows fix, slashes should be always in unix-style
         self.stylesheet = self.stylesheet.replace('\\', '/')
 
@@ -582,7 +595,7 @@ class FadingTipBox(FadingDialog):
 
     def context_menu_requested(self, event):
         pos = QPoint(event.x(), event.y())
-        menu = QMenu(self)
+        menu = SpyderMenu(self)
 
         actions = []
         action_title = create_action(self, _('Go to step: '), icon=QIcon())
@@ -983,7 +996,8 @@ class AnimatedTour(QWidget):
             pass
 
         self.is_running = False
-        self.spy_window.layouts.quick_layout_switch('current_temp')
+        if self.spy_window.layouts:
+            self.spy_window.layouts.quick_layout_switch('current_temp')
         self.spy_window.setUpdatesEnabled(True)
 
     def hide_tips(self):
@@ -1050,7 +1064,7 @@ class AnimatedTour(QWidget):
         return f
 
 
-class OpenTourDialog(QDialog):
+class OpenTourDialog(QDialog, SvgToScaledPixmap):
     """Initial widget with tour."""
 
     def __init__(self, parent, tour_function):
@@ -1061,20 +1075,18 @@ class OpenTourDialog(QDialog):
         else:
             flags = self.windowFlags() & ~Qt.WindowContextHelpButtonHint
         self.setWindowFlags(flags)
+        self.setWindowTitle(_("Spyder tour"))
         self.tour_function = tour_function
 
         # Image
-        images_layout = QHBoxLayout()
-        icon_filename = 'tour-spyder-logo'
-        image_path = get_image_path(icon_filename)
-        image = QPixmap(image_path)
-        image_label = QLabel()
-        image_height = int(image.height() * DialogStyle.IconScaleFactor)
-        image_width = int(image.width() * DialogStyle.IconScaleFactor)
-        image = image.scaled(image_width, image_height, Qt.KeepAspectRatio,
-                             Qt.SmoothTransformation)
-        image_label.setPixmap(image)
+        image_label = QLabel(self)
+        image_label.setPixmap(
+            self.svg_to_scaled_pixmap(
+                "tour-spyder-logo", rescale=DialogStyle.IconScaleFactor
+            )
+        )
 
+        images_layout = QHBoxLayout()
         images_layout.addStretch()
         images_layout.addWidget(image_label)
         images_layout.addStretch()
@@ -1085,71 +1097,29 @@ class OpenTourDialog(QDialog):
 
         # Label
         tour_label_title = QLabel(_("Welcome to Spyder!"))
-        tour_label_title.setStyleSheet(f"font-size: {DialogStyle.TitleFontSize}")
+        tour_label_title.setObjectName("title")
         tour_label_title.setWordWrap(True)
         tour_label = QLabel(
             _("Check out our interactive tour to "
               "explore some of Spyder's panes and features."))
-        tour_label.setStyleSheet(f"font-size: {DialogStyle.ContentFontSize}")
+        tour_label.setObjectName("content")
         tour_label.setWordWrap(True)
         tour_label.setFixedWidth(340)
 
         # Buttons
-        buttons_layout = QHBoxLayout()
-        dialog_tour_color = QStylePalette.COLOR_BACKGROUND_2
-        start_tour_color = QStylePalette.COLOR_ACCENT_2
-        start_tour_hover = QStylePalette.COLOR_ACCENT_3
-        start_tour_pressed = QStylePalette.COLOR_ACCENT_4
-        dismiss_tour_color = QStylePalette.COLOR_BACKGROUND_4
-        dismiss_tour_hover = QStylePalette.COLOR_BACKGROUND_5
-        dismiss_tour_pressed = QStylePalette.COLOR_BACKGROUND_6
-        font_color = QStylePalette.COLOR_TEXT_1
         self.launch_tour_button = QPushButton(_('Start tour'))
-        self.launch_tour_button.setStyleSheet((
-          "QPushButton {{ "
-          "background-color: {background_color};"
-          "border-color: {border_color};"
-          "font-size: {font_size};"
-          "color: {font_color};"
-          "padding: {padding}}}"
-          "QPushButton:hover:!pressed {{ "
-          "background-color: {color_hover}}}"
-          "QPushButton:pressed {{ "
-          "background-color: {color_pressed}}}"
-        ).format(background_color=start_tour_color,
-                 border_color=start_tour_color,
-                 font_size=DialogStyle.ButtonsFontSize,
-                 font_color=font_color,
-                 padding=DialogStyle.ButtonsPadding,
-                 color_hover=start_tour_hover,
-                 color_pressed=start_tour_pressed))
+        self.launch_tour_button.setObjectName("launch-button")
         self.launch_tour_button.setAutoDefault(False)
+
         self.dismiss_button = QPushButton(_('Dismiss'))
-        self.dismiss_button.setStyleSheet((
-          "QPushButton {{ "
-          "background-color: {background_color};"
-          "border-color: {border_color};"
-          "font-size: {font_size};"
-          "color: {font_color};"
-          "padding: {padding}}}"
-          "QPushButton:hover:!pressed {{ "
-          "background-color: {color_hover}}}"
-          "QPushButton:pressed {{ "
-          "background-color: {color_pressed}}}"
-        ).format(background_color=dismiss_tour_color,
-                 border_color=dismiss_tour_color,
-                 font_size=DialogStyle.ButtonsFontSize,
-                 font_color=font_color,
-                 padding=DialogStyle.ButtonsPadding,
-                 color_hover=dismiss_tour_hover,
-                 color_pressed=dismiss_tour_pressed))
+        self.dismiss_button.setObjectName("dismiss-button")
         self.dismiss_button.setAutoDefault(False)
 
-        buttons_layout.addStretch()
-        buttons_layout.addWidget(self.launch_tour_button)
-        if not MAC:
-            buttons_layout.addSpacing(10)
-        buttons_layout.addWidget(self.dismiss_button)
+        bbox = SpyderDialogButtonBox()
+        bbox.addButton(self.launch_tour_button, QDialogButtonBox.ActionRole)
+        bbox.addButton(self.dismiss_button, QDialogButtonBox.RejectRole)
+        bbox.layout().setSpacing(3 * AppStyle.MarginSize)
+        bbox.setStyleSheet(self._buttons_stylesheet)
 
         layout = QHBoxLayout()
         layout.addLayout(images_layout)
@@ -1168,11 +1138,11 @@ class OpenTourDialog(QDialog):
             vertical_layout.addStretch()
             vertical_layout.addLayout(label_layout)
             vertical_layout.addSpacing(20)
-            vertical_layout.addLayout(buttons_layout)
+            vertical_layout.addWidget(bbox)
             vertical_layout.addStretch()
         else:
             vertical_layout.addLayout(label_layout)
-            vertical_layout.addLayout(buttons_layout)
+            vertical_layout.addWidget(bbox)
 
         general_layout = QHBoxLayout()
         if not MAC:
@@ -1186,18 +1156,81 @@ class OpenTourDialog(QDialog):
             general_layout.addLayout(vertical_layout)
 
         self.setLayout(general_layout)
+        self.setStyleSheet(self._main_stylesheet)
 
         self.launch_tour_button.clicked.connect(self._start_tour)
         self.dismiss_button.clicked.connect(self.close)
-        self.setStyleSheet(f"background-color:{dialog_tour_color}")
         self.setContentsMargins(18, 40, 18, 40)
-        if not MAC:
+        if MAC:
+            general_layout.setSizeConstraint(QLayout.SetFixedSize)
+        else:
             self.setFixedSize(640, 280)
 
     def _start_tour(self):
         self.close()
         self.tour_function()
 
+    @property
+    def _main_stylesheet(self):
+        css = qstylizer.style.StyleSheet()
+
+        # Set background color
+        for widget in ["QDialog", "QLabel"]:
+            css[widget].setValues(
+                backgroundColor=DialogStyle.BackgroundColor
+            )
+
+        # Set font size for QLabels
+        css["QLabel#title"].setValues(
+            fontSize=DialogStyle.TitleFontSize
+        )
+
+        css["QLabel#content"].setValues(
+            fontSize=DialogStyle.ContentFontSize
+        )
+
+        return css.toString()
+
+    @property
+    def _buttons_stylesheet(self):
+        css = qstylizer.style.StyleSheet()
+
+        # Set base style for buttons
+        css.QPushButton.setValues(
+            color=SpyderPalette.COLOR_TEXT_1,
+            fontSize=DialogStyle.ButtonsFontSize,
+            padding=DialogStyle.ButtonsPadding,
+        )
+
+        # Set style for launch button
+        css["QPushButton#launch-button"].setValues(
+            backgroundColor=SpyderPalette.COLOR_ACCENT_2,
+            borderColor=SpyderPalette.COLOR_ACCENT_2,
+        )
+
+        css["QPushButton#launch-button:hover:!pressed"].setValues(
+            backgroundColor=SpyderPalette.COLOR_ACCENT_3,
+        )
+
+        css["QPushButton#launch-button:pressed"].setValues(
+            backgroundColor=SpyderPalette.COLOR_ACCENT_4,
+        )
+
+        # Set style for dismiss button
+        css["QPushButton#dismiss-button"].setValues(
+            backgroundColor=SpyderPalette.COLOR_BACKGROUND_4,
+            borderColor=SpyderPalette.COLOR_BACKGROUND_4,
+        )
+
+        css["QPushButton#dismiss-button:hover:!pressed"].setValues(
+            backgroundColor=SpyderPalette.COLOR_BACKGROUND_5,
+        )
+
+        css["QPushButton#dismiss-button:pressed"].setValues(
+            backgroundColor=SpyderPalette.COLOR_BACKGROUND_6,
+        )
+
+        return css.toString()
 
 # ----------------------------------------------------------------------------
 # Used for testing the functionality
