@@ -11,12 +11,14 @@ Tests for the array editor.
 """
 
 # Standard library imports
+from dataclasses import dataclass
 import datetime
+from unittest.mock import patch
 
 # Third party imports
-from qtpy.QtCore import Qt
 import numpy as np
 import pytest
+from qtpy.QtCore import Qt
 
 # Local imports
 from spyder.config.manager import CONF
@@ -172,5 +174,108 @@ def test_objectexplorer_types(objectexplorer, params):
     assert model.columnCount() == 11
 
 
+@dataclass
+class DataclassForTesting:
+    name: str
+    price: float
+    quantity: int
+
+
+def test_objectexplorer_refreshaction_disabled():
+    """
+    Test that the refresh action is disabled by default.
+    """
+    data = DataclassForTesting('lemon', 0.15, 5)
+    editor = ObjectExplorer(data, name='data')
+    assert not editor.refresh_action.isEnabled()
+
+
+def test_objectexplorer_refresh():
+    """
+    Test that after pressing the refresh button, the value of the editor is
+    replaced by the return value of the data_function.
+    """
+    data_old = DataclassForTesting('lemon', 0.15, 5)
+    data_new = range(1, 42, 3)
+    editor = ObjectExplorer(data_old, name='data',
+                            data_function=lambda: data_new)
+    model = editor.obj_tree.model()
+    root = model.index(0, 0)
+    assert model.data(model.index(0, 0, root), Qt.DisplayRole) == 'name'
+    assert model.data(model.index(0, 3, root), Qt.DisplayRole) == 'lemon'
+    assert editor.refresh_action.isEnabled()
+    editor.refresh_editor()
+    model = editor.obj_tree.model()
+    root = model.index(0, 0)
+    row = model.rowCount(root) - 1
+    assert model.data(model.index(row, 0, root), Qt.DisplayRole) == 'stop'
+    assert model.data(model.index(row, 3, root), Qt.DisplayRole) == '42'
+
+
+def test_objectexplorer_refresh_when_variable_deleted(qtbot):
+    """
+    Test that if the variable is deleted and then the editor is refreshed
+    (resulting in data_function raising a KeyError), a critical dialog box
+    is displayed and that the object editor is closed.
+    """
+    def datafunc():
+        raise KeyError
+    data = DataclassForTesting('lemon', 0.15, 5)
+    editor = ObjectExplorer(data, name='data', data_function=datafunc)
+    with patch('spyder.plugins.variableexplorer.widgets.objectexplorer'
+               '.objectexplorer.QMessageBox.critical') as mock_critical:
+        with qtbot.waitSignal(editor.rejected, timeout=0):
+            editor.refresh_action.trigger()
+    mock_critical.assert_called_once()
+
+
+@dataclass
+class Box:
+    contents: object
+
+
+def test_objectexplorer_refresh_nested():
+    """
+    Open an editor for a `Box` object containing a list, and then open another
+    editor for the nested list. Test that refreshing the second editor works.
+    """
+    old_data = Box([1, 2, 3])
+    new_data = Box([4, 5])
+    editor = ObjectExplorer(
+        old_data, name='data', data_function=lambda: new_data)
+    model = editor.obj_tree.model()
+    root_index = model.index(0, 0)
+    contents_index = model.index(0, 0, root_index)
+    editor.obj_tree.edit(contents_index)
+    delegate = editor.obj_tree.delegate
+    nested_editor = list(delegate._editors.values())[0]['editor']
+    assert nested_editor.get_value() == [1, 2, 3]
+    nested_editor.widget.refresh_action.trigger()
+    assert nested_editor.get_value() == [4, 5]
+
+
+def test_objectexplorer_refresh_doubly_nested():
+    """
+    Open an editor for a `Box` object containing another `Box` object which
+    in turn contains a list. Then open a second editor for the nested list.
+    Test that refreshing the second editor works.
+    """
+    old_data = Box(Box([1, 2, 3]))
+    new_data = Box(Box([4, 5]))
+    editor = ObjectExplorer(
+        old_data, name='data', data_function=lambda: new_data)
+    model = editor.obj_tree.model()
+    root_index = model.index(0, 0)
+    inner_box_index = model.index(0, 0, root_index)
+    editor.obj_tree.expand(inner_box_index)
+    contents_index = model.index(0, 0, inner_box_index)
+    editor.obj_tree.edit(contents_index)
+    delegate = editor.obj_tree.delegate
+    nested_editor = list(delegate._editors.values())[0]['editor']
+    assert nested_editor.get_value() == [1, 2, 3]
+    nested_editor.widget.refresh_action.trigger()
+    assert nested_editor.get_value() == [4, 5]
+
+
 if __name__ == "__main__":
     pytest.main()
