@@ -9,7 +9,7 @@ Tests for namespacebrowser.py
 
 # Standard library imports
 import string
-from unittest.mock import Mock
+from unittest.mock import Mock, patch
 
 # Third party imports
 from flaky import flaky
@@ -18,7 +18,7 @@ from qtpy.QtCore import Qt, QPoint, QModelIndex
 
 # Local imports
 from spyder.plugins.variableexplorer.widgets.namespacebrowser import (
-    NamespaceBrowser, NamespacesBrowserFinder, VALID_VARIABLE_CHARS)
+    NamespaceBrowser)
 from spyder.widgets.collectionseditor import ROWS_TO_LOAD
 from spyder.widgets.tests.test_collectioneditor import data, data_table
 
@@ -155,13 +155,6 @@ def test_filtering_with_large_rows(namespacebrowser, qtbot):
     """
     browser = namespacebrowser
 
-    text_finder = NamespacesBrowserFinder(
-        browser.editor,
-        callback=browser .editor.set_regex,
-        main=browser,
-        regex_base=VALID_VARIABLE_CHARS)
-    browser.set_text_finder(text_finder)
-
     # Create data
     variables = {}
     for i in range(200):
@@ -183,7 +176,7 @@ def test_filtering_with_large_rows(namespacebrowser, qtbot):
     assert data(model, 49, 0) == 'e49'
 
     # Assert we can filter variables not loaded yet.
-    qtbot.keyClicks(text_finder, "t19")
+    browser.do_find("t19")
     assert model.rowCount() == 10
 
     # Assert all variables effectively start with 't19'.
@@ -191,7 +184,7 @@ def test_filtering_with_large_rows(namespacebrowser, qtbot):
         assert data(model, i, 0) == 't19{}'.format(i)
 
     # Reset text_finder widget.
-    text_finder.setText('')
+    browser.do_find('')
 
     # Create a new variable that starts with a different letter than
     # the rest.
@@ -206,9 +199,86 @@ def test_filtering_with_large_rows(namespacebrowser, qtbot):
     browser.process_remote_view(new_variables)
 
     # Assert that can find 'z' among the declared variables.
-    qtbot.keyClicks(text_finder, "z")
+    browser.do_find("z")
     assert model.rowCount() == 1
 
 
+def test_namespacebrowser_plot_with_mute_inline_plotting_true(
+        namespacebrowser, qtbot):
+    """
+    Test that plotting a list from the namespace browser sends a signal
+    with the plot if `mute_inline_plotting` is set to `True`.
+    """
+    namespacebrowser.set_conf('mute_inline_plotting', True, section='plots')
+    namespacebrowser.plots_plugin_enabled = True
+    my_list = [4, 2]
+    mock_figure = Mock()
+    mock_axis = Mock()
+    mock_png = b'fake png'
+
+    with patch('spyder.pyplot.subplots',
+               return_value=(mock_figure, mock_axis)), \
+         patch('IPython.core.pylabtools.print_figure',
+               return_value=mock_png) as mock_print_figure, \
+         qtbot.waitSignal(namespacebrowser.sig_show_figure_requested) \
+             as blocker:
+        namespacebrowser.plot(my_list, 'plot')
+
+    mock_axis.plot.assert_called_once_with(my_list)
+    mock_print_figure.assert_called_once_with(
+        mock_figure, fmt='png', bbox_inches='tight', dpi=144)
+    expected_args = [mock_png, 'image/png', namespacebrowser.shellwidget]
+    assert blocker.args == expected_args
+
+
+def test_namespacebrowser_plot_options(namespacebrowser):
+    """
+    Test that font.size and figure.subplot.bottom in matplotlib.rcParams are
+    set to the values from the Spyder preferences when plotting.
+    """
+    def check_rc(*args):
+        from matplotlib import rcParams
+        assert rcParams['font.size'] == 20.5
+        assert rcParams['figure.subplot.bottom'] == 0.314
+
+    namespacebrowser.set_conf('mute_inline_plotting', True, section='plots')
+    namespacebrowser.plots_plugin_enabled = True
+    namespacebrowser.set_conf(
+        'pylab/inline/fontsize', 20.5, section='ipython_console'
+    )
+    namespacebrowser.set_conf(
+        'pylab/inline/bottom', 0.314, section='ipython_console'
+    )
+
+    mock_figure = Mock()
+    mock_axis = Mock()
+    mock_png = b'fake png'
+
+    with patch('spyder.pyplot.subplots',
+               return_value=(mock_figure, mock_axis)), \
+         patch('IPython.core.pylabtools.print_figure',
+               return_value=mock_png), \
+         patch.object(mock_axis, 'plot', check_rc):
+        namespacebrowser.plot([4, 2], 'plot')
+
+
+def test_namespacebrowser_plot_with_mute_inline_plotting_false(namespacebrowser):
+    """
+    Test that plotting a list from the namespace browser shows a plot if
+    `mute_inline_plotting` is set to `False`.
+    """
+    namespacebrowser.set_conf('mute_inline_plotting', False, section='plots')
+    my_list = [4, 2]
+
+    with patch('spyder.pyplot.figure') as mock_figure, \
+         patch('spyder.pyplot.plot') as mock_plot, \
+         patch('spyder.pyplot.show') as mock_show:
+        namespacebrowser.plot(my_list, 'plot')
+
+    mock_figure.assert_called_once_with()
+    mock_plot.assert_called_once_with(my_list)
+    mock_show.assert_called_once_with()
+
+
 if __name__ == "__main__":
     pytest.main()
