@@ -13,8 +13,16 @@ import os
 import os.path as osp
 import sys
 
+# Third-party imports
+from spyder_kernels.utils.pythonenv import (
+    add_quotes,
+    get_conda_env_path,
+    is_conda_env,
+)
+
+# Local imports
 from spyder.utils.programs import find_program, run_program, run_shell_command
-from spyder.config.base import get_spyder_umamba_path
+from spyder.config.base import is_conda_based_app
 
 WINDOWS = os.name == 'nt'
 CONDA_ENV_LIST_CACHE = {}
@@ -38,26 +46,6 @@ def _env_for_conda():
     return env
 
 
-def add_quotes(path):
-    """Return quotes if needed for spaces on path."""
-    quotes = '"' if ' ' in path and '"' not in path else ''
-    return '{quotes}{path}{quotes}'.format(quotes=quotes, path=path)
-
-
-def is_conda_env(prefix=None, pyexec=None):
-    """Check if prefix or python executable are in a conda environment."""
-    if pyexec is not None:
-        pyexec = pyexec.replace('\\', '/')
-
-    if (prefix is None and pyexec is None) or (prefix and pyexec):
-        raise ValueError('Only `prefix` or `pyexec` should be provided!')
-
-    if pyexec and prefix is None:
-        prefix = get_conda_env_path(pyexec).replace('\\', '/')
-
-    return os.path.exists(os.path.join(prefix, 'conda-meta'))
-
-
 def get_conda_root_prefix(pyexec=None, quote=False):
     """
     Return conda prefix from pyexec path
@@ -83,61 +71,33 @@ def get_conda_root_prefix(pyexec=None, quote=False):
     return root_prefix
 
 
-def get_conda_activation_script(quote=False):
+def find_conda(pyexec=None):
     """
-    Return full path to conda activation script.
+    Find conda executable.
 
-    If `quote` is True, then quotes are added if spaces are found in the path.
+    `pyexec` is a python executable, the relative location from which to
+    attempt to locate a conda executable.
     """
-    # Use micromamba bundled with Spyder installers or find conda exe
-    exe = get_spyder_umamba_path() or find_conda()
+    conda = None
 
-    if osp.basename(exe).startswith('micromamba'):
-        # For Micromamba, use the executable
-        script_path = exe
-    else:
-        # Conda activation script is relative to executable
-        conda_exe_root = osp.dirname(osp.dirname(exe))
-        if WINDOWS:
-            activate = 'Scripts/activate'
-        else:
-            activate = 'bin/activate'
-        script_path = osp.join(conda_exe_root, activate)
+    # First try Spyder's conda executable
+    if is_conda_based_app():
+        root = osp.dirname(os.environ['CONDA_EXE'])
+        conda = osp.join(root, 'mamba.exe' if WINDOWS else 'mamba')
 
-    script_path = script_path.replace('\\', '/')
+    # Next try the environment variables
+    if conda is None:
+        conda = os.environ.get('CONDA_EXE') or os.environ.get('MAMBA_EXE')
 
-    if quote:
-        script_path = add_quotes(script_path)
-
-    return script_path
-
-
-def get_conda_env_path(pyexec, quote=False):
-    """
-    Return the full path to the conda environment from give python executable.
-
-    If `quote` is True, then quotes are added if spaces are found in the path.
-    """
-    pyexec = pyexec.replace('\\', '/')
-    if os.name == 'nt':
-        conda_env = os.path.dirname(pyexec)
-    else:
-        conda_env = os.path.dirname(os.path.dirname(pyexec))
-
-    if quote:
-        conda_env = add_quotes(conda_env)
-
-    return conda_env
-
-
-def find_conda():
-    """Find conda executable."""
-    # First try the environment variables
-    conda = os.environ.get('CONDA_EXE') or os.environ.get('MAMBA_EXE')
+    # Next try searching for the executable
     if conda is None:
-        # Try searching for the executable
         conda_exec = 'conda.bat' if WINDOWS else 'conda'
-        conda = find_program(conda_exec)
+        extra_paths = [
+            osp.join(get_conda_root_prefix(_pyexec), 'condabin')
+            for _pyexec in [sys.executable, pyexec]
+        ]
+        conda = find_program(conda_exec, extra_paths)
+
     return conda
 
 
@@ -164,8 +124,12 @@ def get_list_conda_envs():
         path = osp.join(env, 'python.exe') if WINDOWS else osp.join(
             env, 'bin', 'python')
 
-        # In case the environment doesn't have Python
-        if not osp.isfile(path):
+        if (
+            # In case the environment doesn't have Python
+            not osp.isfile(path)
+            # Don't list the installers base env
+            or (is_conda_based_app(pyexec=path) and name != "spyder-runtime")
+        ):
             continue
 
         try:
@@ -176,7 +140,7 @@ def get_list_conda_envs():
 
         name = ('base' if name.lower().startswith('anaconda') or
                 name.lower().startswith('miniconda') else name)
-        name = 'conda: {}'.format(name)
+        name = 'Conda: {}'.format(name)
         env_list[name] = (path, version.strip())
 
     CONDA_ENV_LIST_CACHE = env_list
@@ -215,12 +179,21 @@ def get_spyder_conda_channel():
     except Exception:
         return None, None
 
+    # Avoids iterating over non-dict objects
+    if 'error' in out:
+        return None, None
+
+    # These variables can be unassigned after the next for, so we need to give
+    # them a default value at this point.
+    # Fixes spyder-ide/spyder#22054
+    channel, channel_url = None, None
+
     for package_info in out:
         if package_info["name"] == 'spyder':
             channel = package_info["channel"]
             channel_url = package_info["base_url"]
 
-    if "<develop>" in channel_url:
+    if channel_url is not None and "<develop>" in channel_url:
         channel_url = None
 
     return channel, channel_url
