@@ -8,10 +8,10 @@
 
 # Standard library imports
 from ast import literal_eval
-from getpass import getuser
-from textwrap import dedent
 import glob
+from getpass import getuser
 import importlib
+from importlib.metadata import PackageNotFoundError, version as package_version
 import itertools
 import os
 import os.path as osp
@@ -19,20 +19,23 @@ import re
 import subprocess
 import sys
 import tempfile
+from textwrap import dedent
 import threading
 import time
+import logging
 
 # Third party imports
 from packaging.version import parse
-import pkg_resources
 import psutil
+from spyder_kernels.utils.pythonenv import is_conda_env
 
 # Local imports
-from spyder.config.base import (running_under_pytest, get_home_dir,
-                                running_in_mac_app)
+from spyder.config.base import _, running_under_pytest, get_home_dir
 from spyder.utils import encoding
 from spyder.utils.misc import get_python_executable
 
+
+logger = logging.getLogger(__name__)
 HERE = osp.abspath(osp.dirname(__file__))
 
 
@@ -68,7 +71,7 @@ def get_temp_dir(suffix=None):
     return tempdir
 
 
-def is_program_installed(basename):
+def is_program_installed(basename, extra_paths=[]):
     """
     Return program absolute path if installed in PATH.
     Otherwise, return None.
@@ -76,7 +79,7 @@ def is_program_installed(basename):
     Also searches specific platform dependent paths that are not already in
     PATH. This permits general use without assuming user profiles are
     sourced (e.g. .bash_Profile), such as when login shells are not used to
-    launch Spyder.
+    launch Spyder. Additionally, extra_paths are searched.
 
     On macOS systems, a .app is considered installed if it exists.
     """
@@ -107,15 +110,15 @@ def is_program_installed(basename):
              'Miniconda3', 'Anaconda3', 'Miniconda', 'Anaconda']
 
     conda = [osp.join(*p, 'condabin') for p in itertools.product(a, b)]
-    req_paths.extend(pyenv + conda)
+    req_paths.extend(pyenv + conda + extra_paths)
 
-    for path in os.environ['PATH'].split(os.pathsep) + req_paths:
+    for path in set(os.environ['PATH'].split(os.pathsep) + req_paths):
         abspath = osp.join(path, basename)
         if osp.isfile(abspath):
             return abspath
 
 
-def find_program(basename):
+def find_program(basename, extra_paths=[]):
     """
     Find program in PATH and return absolute path
 
@@ -129,7 +132,7 @@ def find_program(basename):
         if not basename.endswith(extensions):
             names = [basename + ext for ext in extensions] + [basename]
     for name in names:
-        path = is_program_installed(name)
+        path = is_program_installed(name, extra_paths)
         if path:
             return path
 
@@ -175,7 +178,7 @@ def alter_subprocess_kwargs_by_platform(**kwargs):
         # ensure Windows subprocess environment has SYSTEMROOT
         if kwargs.get('env') is not None:
             # Is SYSTEMROOT, SYSTEMDRIVE in env? case insensitive
-            for env_var in ['SYSTEMROOT', 'SYSTEMDRIVE']:
+            for env_var in ['SYSTEMROOT', 'SYSTEMDRIVE', 'USERPROFILE']:
                 if env_var not in map(str.upper, kwargs['env'].keys()):
                     # Add from os.environ
                     for k, v in os.environ.items():
@@ -705,6 +708,122 @@ def get_python_args(fname, python_args, interact, debug, end_args):
     return p_args
 
 
+def run_general_file_in_terminal(
+    executable: str,
+    args: str,
+    fname: str,
+    script_args: str,
+    wdir: str,
+    close_after_exec: bool = False,
+    windows_shell: str = "cmd.exe /K"
+):
+    """
+    Run a file on a given CLI executable.
+
+    Arguments
+    ---------
+    executable: str
+        Name or path to the executable.
+    args: str
+        Arguments to pass to the executable.
+    fname: str
+        Path to the file to execute in the shell interpreter/executable.
+    script_args: str
+        File arguments
+    wdir: str
+        Working directory path from which the file will be executed.
+    windows_shell: str
+        Name of the executable to use as shell (Windows only).
+    """
+
+    # Quote fname in case it has spaces (all platforms)
+    # fname = f'"{fname}"'
+    wdir = None if not wdir else wdir  # Cannot be empty string
+    args = shell_split(args)
+    script_args = shell_split(script_args)
+    p_args = args + [fname] + script_args
+    p_args = [f'"{x}"' for x in p_args]
+
+    if os.name == 'nt':
+        if wdir is not None:
+            # wdir can come with / as os.sep, so we need to take care of it.
+            wdir = wdir.replace('/', '\\')
+
+        # python_exe must be quoted in case it has spaces
+        cmd = f'start {windows_shell} ""{executable}" '
+        cmd += ' '.join(p_args)  # + '"'
+        logger.info('Executing on external console: %s', cmd)
+
+        try:
+            run_shell_command(cmd, cwd=wdir)
+        except WindowsError:
+            from qtpy.QtWidgets import QMessageBox
+            QMessageBox.critical(
+                None,
+                _('Run'),
+                _("It was not possible to run this file in an external "
+                  "terminal"),
+                QMessageBox.Ok
+            )
+    elif sys.platform.startswith('linux'):
+        programs = [{'cmd': 'gnome-terminal', 'execute-option': '--'},
+                    {'cmd': 'konsole', 'execute-option': '-e'},
+                    {'cmd': 'xfce4-terminal', 'execute-option': '-x'},
+                    {'cmd': 'xterm', 'execute-option': '-e'}]
+        for program in programs:
+            if is_program_installed(program['cmd']):
+                f = None
+                if not close_after_exec:
+                    f = tempfile.NamedTemporaryFile(
+                        'wt',
+                        prefix='run_spyder_',
+                        suffix='.sh',
+                        dir=get_temp_dir(),
+                        delete=False
+                    )
+
+                    logger.info('Executing on external console: %s',
+                                ' '.join([executable] + p_args))
+                    f.write(' '.join([executable] + p_args) + '\n')
+                    f.write(f'read -p "{_("Press enter to continue...")}"\n')
+                    executable = '/usr/bin/bash'
+                    p_args = [f.name]
+
+                cmd = [program['cmd'], program['execute-option'], executable]
+                cmd.extend(p_args)
+                run_shell_command(' '.join(cmd), cwd=wdir)
+                if f:
+                    f.close()
+                return
+    elif sys.platform == 'darwin':
+        f = tempfile.NamedTemporaryFile(
+            'wt',
+            prefix='run_spyder_',
+            suffix='.sh',
+            dir=get_temp_dir(),
+            delete=False
+        )
+        if wdir:
+            f.write('cd "{}"\n'.format(wdir))
+        f.write(' '.join([executable] + p_args) + '\n')
+        if not close_after_exec:
+            f.write(f'read -p "{_("Press enter to continue...")}"\n')
+        f.close()
+        os.chmod(f.name, 0o777)
+
+        def run_terminal_thread():
+            proc = run_shell_command(f'open -a Terminal.app {f.name}')
+            # Prevent race condition
+            time.sleep(3)
+            proc.wait()
+            os.remove(f.name)
+
+        thread = threading.Thread(target=run_terminal_thread)
+        thread.start()
+    else:
+        raise NotImplementedError
+
+
 def run_python_script_in_terminal(fname, wdir, args, interact, debug,
                                   python_args, executable=None, pypath=None):
     """
@@ -736,7 +855,6 @@ def run_python_script_in_terminal(fname, wdir, args, interact, debug,
             # UNC paths start with \\
             if osp.splitdrive(wdir)[0].startswith("\\\\"):
                 from qtpy.QtWidgets import QMessageBox
-                from spyder.config.base import _
                 QMessageBox.critical(
                     None,
                     _('Run'),
@@ -753,7 +871,6 @@ def run_python_script_in_terminal(fname, wdir, args, interact, debug,
             run_shell_command(cmd, cwd=wdir, env=env)
         except WindowsError:
             from qtpy.QtWidgets import QMessageBox
-            from spyder.config.base import _
             QMessageBox.critical(
                 None,
                 _('Run'),
@@ -762,7 +879,7 @@ def run_python_script_in_terminal(fname, wdir, args, interact, debug,
                 QMessageBox.Ok
             )
     elif sys.platform.startswith('linux'):
-        programs = [{'cmd': 'gnome-terminal', 'execute-option': '-x'},
+        programs = [{'cmd': 'gnome-terminal', 'execute-option': '--'},
                     {'cmd': 'konsole', 'execute-option': '-e'},
                     {'cmd': 'xfce4-terminal', 'execute-option': '-x'},
                     {'cmd': 'xterm', 'execute-option': '-e'}]
@@ -778,8 +895,6 @@ def run_python_script_in_terminal(fname, wdir, args, interact, debug,
                                         delete=False)
         if wdir:
             f.write('cd "{}"\n'.format(wdir))
-        if running_in_mac_app(executable):
-            f.write(f'export PYTHONHOME={os.environ["PYTHONHOME"]}\n')
         if pypath is not None:
             f.write(f'export PYTHONPATH={pypath}\n')
         f.write(' '.join([executable] + p_args))
@@ -799,6 +914,32 @@ def run_python_script_in_terminal(fname, wdir, args, interact, debug,
         raise NotImplementedError
 
 
+def check_version_range(module_version, version_range):
+    """
+    Check if a module's version lies in `version_range`.
+    """
+    if ',' in version_range:
+        versions = version_range.split(',')
+    else:
+        versions = [version_range]
+
+    output = True
+    for _ver in versions:
+        match = re.search(r'[0-9]', _ver)
+        assert match is not None, "Invalid version number"
+        symb = _ver[:match.start()]
+        if not symb:
+            symb = '='
+
+        if symb not in ['>=', '>', '=', '<', '<=', '!=']:
+            raise RuntimeError(f"Invalid version condition '{symb}'")
+
+        ver = _ver[match.start():]
+        output = output and check_version(module_version, ver, symb)
+
+    return output
+
+
 def check_version(actver, version, cmp_op):
     """
     Check version string of an active module against a required version.
@@ -834,8 +975,32 @@ def check_version(actver, version, cmp_op):
         return True
 
 
-def get_module_version(module_name):
+def get_module_version(module_name, interpreter=None):
     """Return module version or None if version can't be retrieved."""
+    if interpreter:
+        cmd = dedent("""
+            try:
+                import {} as mod
+            except Exception:
+                print('No Module')  # spyder: test-skip
+            print(
+                getattr(mod, '__version__', getattr(mod, 'VERSION', None))
+            )  # spyder: test-skip
+            """
+        ).format(module_name)
+
+        # Use clean environment
+        proc = run_program(interpreter, ['-c', cmd], env={})
+        stdout, stderr = proc.communicate()
+        stdout = stdout.decode().strip()
+
+        if 'No Module' in stdout:
+            raise RuntimeError("No module named " + str(module_name))
+        if stdout != 'None':
+            # the module is installed and it has a version attribute
+            return stdout
+        return None
+
     mod = __import__(module_name)
     ver = getattr(mod, '__version__', getattr(mod, 'VERSION', None))
     if not ver:
@@ -845,13 +1010,9 @@ def get_module_version(module_name):
 
 def get_package_version(package_name):
     """Return package version or None if version can't be retrieved."""
-
-    # When support for Python 3.7 and below is dropped, this can be replaced
-    # with the built-in importlib.metadata.version
     try:
-        ver = pkg_resources.get_distribution(package_name).version
-        return ver
-    except pkg_resources.DistributionNotFound:
+        return package_version(package_name)
+    except PackageNotFoundError:
         return None
 
 
@@ -876,28 +1037,10 @@ def is_module_installed(module_name, version=None, interpreter=None,
     """
     if interpreter is not None:
         if is_python_interpreter(interpreter):
-            cmd = dedent("""
-                try:
-                    import {} as mod
-                except Exception:
-                    print('No Module')  # spyder: test-skip
-                print(getattr(mod, '__version__', getattr(mod, 'VERSION', None)))  # spyder: test-skip
-                """).format(module_name)
             try:
-                # use clean environment
-                proc = run_program(interpreter, ['-c', cmd], env={})
-                stdout, stderr = proc.communicate()
-                stdout = stdout.decode().strip()
+                module_version = get_module_version(module_name, interpreter)
             except Exception:
                 return False
-
-            if 'No Module' in stdout:
-                return False
-            elif stdout != 'None':
-                # the module is installed and it has a version attribute
-                module_version = stdout
-            else:
-                module_version = None
         else:
             # Try to not take a wrong decision if interpreter check fails
             return True
@@ -927,26 +1070,7 @@ def is_module_installed(module_name, version=None, interpreter=None,
     if version is None:
         return True
     else:
-        if ',' in version:
-            versions = version.split(',')
-        else:
-            versions = [version]
-
-        output = True
-        for _ver in versions:
-            match = re.search(r'[0-9]', _ver)
-            assert match is not None, "Invalid version number"
-            symb = _ver[:match.start()]
-            if not symb:
-                symb = '='
-
-            if symb not in ['>=', '>', '=', '<', '<=', '!=']:
-                raise RuntimeError(f"Invalid version condition '{symb}'")
-
-            ver = _ver[match.start():]
-            output = output and check_version(module_version, ver, symb)
-
-        return output
+        return check_version_range(module_version, version)
 
 
 def is_python_interpreter_valid_name(filename):
@@ -960,9 +1084,6 @@ def is_python_interpreter_valid_name(filename):
 
 def is_python_interpreter(filename):
     """Evaluate whether a file is a python interpreter or not."""
-    # Must be imported here to avoid circular import
-    from spyder.utils.conda import is_conda_env
-
     real_filename = os.path.realpath(filename)  # To follow symlink if existent
 
     if (not osp.isfile(real_filename) or
@@ -1041,7 +1162,8 @@ def is_spyder_process(pid):
 
         # Valid names for main script
         names = set(['spyder', 'spyder3', 'spyder.exe', 'spyder3.exe',
-                     'bootstrap.py', 'spyder-script.py', 'Spyder.launch.pyw'])
+                     'bootstrap.py', 'spyder-script.py', 'spyder-script.pyw',
+                     'Spyder.launch.pyw'])
         if running_under_pytest():
             names.add('runtests.py')
 
