@@ -13,6 +13,7 @@
 
 # Standard library imports
 import re
+import sys
 
 # Third party imports
 from qtpy.QtCore import QEvent, QSize, Qt, QTimer, Signal, Slot
@@ -22,14 +23,15 @@ from qtpy.QtWidgets import (QAction, QGridLayout, QHBoxLayout, QLabel,
                             QWidget)
 
 # Local imports
+from spyder.api.shortcuts import SpyderShortcutsMixin
 from spyder.config.base import _
-from spyder.config.manager import CONF
 from spyder.py3compat import to_text_string
 from spyder.utils.icon_manager import ima
 from spyder.utils.misc import regexp_error_msg
 from spyder.plugins.editor.utils.editor import TextHelper
 from spyder.utils.qthelpers import create_toolbutton
 from spyder.utils.sourcecode import get_eol_chars
+from spyder.utils.stylesheet import AppStyle
 from spyder.widgets.comboboxes import PatternComboBox
 
 
@@ -45,16 +47,20 @@ def is_position_inf(pos1, pos2):
 class SearchText(PatternComboBox):
 
     def __init__(self, parent):
-        self.recommended_width = 400
+        self.recommended_width = AppStyle.FindMinWidth
         super().__init__(parent, adjust_to_minimum=False)
 
     def sizeHint(self):
         """Recommended size."""
-        return QSize(self.recommended_width, 10)
+        return QSize(self.recommended_width, AppStyle.FindHeight)
 
 
-class FindReplace(QWidget):
+class FindReplace(QWidget, SpyderShortcutsMixin):
     """Find widget"""
+
+    # For shortcuts
+    CONF_SECTION = 'find_replace'
+
     TOOLTIP = {
         'regexp_error': _("Regular expression error"),
         'no_matches': _("No matches")
@@ -71,7 +77,15 @@ class FindReplace(QWidget):
         self.is_code_editor = None
 
         glayout = QGridLayout()
-        glayout.setContentsMargins(6, 3, 6, 3)
+        glayout.setContentsMargins(
+            2 * AppStyle.MarginSize,
+            AppStyle.MarginSize,
+            2 * AppStyle.MarginSize,
+            0
+        )
+        if sys.platform == "darwin":
+            # Spacing is too big on Mac, which makes the widget look bad
+            glayout.setSpacing(2 * AppStyle.MarginSize)
         self.setLayout(glayout)
 
         self.close_button = create_toolbutton(
@@ -104,7 +118,9 @@ class FindReplace(QWidget):
         self.search_text.sig_resized.connect(self._resize_replace_text)
 
         self.number_matches_text = QLabel(self)
-        self.search_text.clear_action.triggered.connect(self.clear_matches)
+        self.search_text.lineEdit().clear_action.triggered.connect(
+            self.clear_matches
+        )
         self.hide_number_matches_text = False
         self.number_matches_pixmap = (
             ima.icon('number_matches').pixmap(self.icon_size)
@@ -249,7 +265,7 @@ class FindReplace(QWidget):
         # Additional adjustments
         self.search_text.setTabOrder(self.search_text, self.replace_text)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
-        self.shortcuts = self.create_shortcuts(parent)
+        self.register_shortcuts(parent)
 
         # To highlight found results in the editor
         self.highlight_timer = QTimer(self)
@@ -307,49 +323,20 @@ class FindReplace(QWidget):
 
         return super().eventFilter(widget, event)
 
-    def create_shortcuts(self, parent):
-        """Create shortcuts for this widget"""
-        # Configurable
-        findnext = CONF.config_shortcut(
-            self.find_next,
-            context='find_replace',
-            name='Find next',
-            parent=parent)
+    def register_shortcuts(self, parent):
+        """Register shortcuts for this widget."""
+        shortcuts = (
+            ('find next', self.find_next, parent),
+            ("find previous", self.find_previous, parent),
+            ('find text', self.show, parent),
+            ('replace text', self.show_replace, parent),
+            ('hide find and replace', self.hide, self),
+        )
 
-        findprev = CONF.config_shortcut(
-            self.find_previous,
-            context='find_replace',
-            name='Find previous',
-            parent=parent)
-
-        togglefind = CONF.config_shortcut(
-            self.show,
-            context='find_replace',
-            name='Find text',
-            parent=parent)
-
-        togglereplace = CONF.config_shortcut(
-            self.show_replace,
-            context='find_replace',
-            name='Replace text',
-            parent=parent)
-
-        hide = CONF.config_shortcut(
-            self.hide,
-            context='find_replace',
-            name='hide find and replace',
-            parent=self)
-
-        return [findnext, findprev, togglefind, togglereplace, hide]
-
-    def get_shortcut_data(self):
-        """
-        Returns shortcut data, a list of tuples (shortcut, text, default)
-        shortcut (QShortcut or QAction instance)
-        text (string): action/shortcut description
-        default (string): default key sequence
-        """
-        return [sc.data for sc in self.shortcuts]
+        for name, callback, widget in shortcuts:
+            self.register_shortcut_for_widget(
+                name=name, triggered=callback, widget=widget
+            )
 
     def update_search_combo(self):
         self.search_text.lineEdit().returnPressed.emit()
@@ -654,7 +641,6 @@ class FindReplace(QWidget):
                 cursor.setPosition(found_cursor.selectionEnd(),
                                    QTextCursor.KeepAnchor)
 
-
         if cursor is not None:
             cursor.endEditBlock()
 
@@ -674,28 +660,33 @@ class FindReplace(QWidget):
 
         replace_text = str(self.replace_text.currentText())
         search_text = str(self.search_text.currentText())
-        re_pattern = None
         case = self.case_button.isChecked()
+        word = self.words_button.isChecked()
         re_flags = re.MULTILINE if case else re.IGNORECASE | re.MULTILINE
-        re_enabled = self.re_button.isChecked()
 
-        # Escape backslashes present in replace_text to avoid an error when
-        # using the regexp pattern below to perform the substitution.
-        # Fixes spyder-ide/spyder#21007.
-        replace_text = replace_text.replace('\\', r'\\')
+        if self.re_button.isChecked():
+            pattern = search_text
+        else:
+            pattern = re.escape(search_text)
+            # re.sub processes backslashes so they must be escaped
+            # See spyder-ide/spyder#21007.
+            replace_text = replace_text.replace('\\', r'\\')
+
+        # Match whole words only
+        if word:
+            pattern = r'\b{pattern}\b'.format(pattern=pattern)
 
         # Check regexp before proceeding
-        if re_enabled:
-            try:
-                re_pattern = re.compile(search_text, flags=re_flags)
-                # Check if replace_text can be substituted in re_pattern
-                # Fixes spyder-ide/spyder#7177.
-                re_pattern.sub(replace_text, '')
-            except re.error:
-                # Do nothing with an invalid regexp
-                return
-        else:
-            re_pattern = re.compile(re.escape(search_text), flags=re_flags)
+        re_pattern = None
+        try:
+            re_pattern = re.compile(pattern, flags=re_flags)
+
+            # Check if replace_text can be substituted in re_pattern
+            # Fixes spyder-ide/spyder#7177.
+            re_pattern.sub(replace_text, '')
+        except re.error:
+            # Do nothing with an invalid regexp
+            return
 
         cursor = self.editor._select_text("sof", "eof")
         text = self.editor.toPlainText()
@@ -721,6 +712,7 @@ class FindReplace(QWidget):
                 pattern = search_text
             else:
                 pattern = re.escape(search_text)
+                # re.sub processes backslashes so they must be escaped
                 replace_text = replace_text.replace('\\', r'\\')
             if word:  # match whole words only
                 pattern = r'\b{pattern}\b'.format(pattern=pattern)
