@@ -16,23 +16,23 @@ from os import path
 import copy
 import datetime
 from xml.dom.minidom import parseString
-from unittest.mock import Mock
+from unittest.mock import Mock, patch
 
 # Third party imports
 import numpy
+from packaging.version import parse
 import pandas
 import pytest
 from flaky import flaky
 from qtpy.QtCore import Qt, QPoint
-from qtpy.QtWidgets import QWidget, QDateEdit
+from qtpy.QtWidgets import QDateEdit, QMessageBox, QWidget
 
 # Local imports
 from spyder.config.manager import CONF
 from spyder.widgets.collectionseditor import (
-    RemoteCollectionsEditorTableView, CollectionsEditorTableView,
-    CollectionsModel, CollectionsEditor, LARGE_NROWS, ROWS_TO_LOAD, natsort)
-from spyder.plugins.variableexplorer.widgets.namespacebrowser import (
-    NamespacesBrowserFinder)
+    CollectionsEditor, CollectionsEditorTableView, CollectionsEditorWidget,
+    CollectionsModel, LARGE_NROWS, natsort, RemoteCollectionsEditorTableView,
+    ROWS_TO_LOAD)
 from spyder.plugins.variableexplorer.widgets.tests.test_dataframeeditor import (
     generate_pandas_indexes)
 from spyder_kernels.utils.nsview import get_size
@@ -244,30 +244,49 @@ def test_filter_rows(qtbot):
              'numpy_type': 'Unknown'}}
     )
     editor = RemoteCollectionsEditorTableView(None, data)
-    editor.finder = NamespacesBrowserFinder(
-        editor, editor.set_regex)
     qtbot.addWidget(editor)
 
     # Initially two rows
     assert editor.model.rowCount() == 2
 
     # Match two rows by name
-    editor.finder.setText("df")
+    editor.do_find("df")
     assert editor.model.rowCount() == 2
 
     # Match two rows by type
-    editor.finder.setText("DataFrame")
+    editor.do_find("DataFrame")
     assert editor.model.rowCount() == 2
 
     # Only one match
-    editor.finder.setText("dfb")
+    editor.do_find("dfb")
     assert editor.model.rowCount() == 1
 
     # No match
-    editor.finder.setText("dfbc")
+    editor.do_find("dfbc")
     assert editor.model.rowCount() == 0
 
 
+def test_remote_make_data_function():
+    """
+    Test that the function returned by make_data_function() is the expected
+    one.
+    """
+    variables = {'a': {'type': 'int',
+                       'size': 1,
+                       'view': '1',
+                       'python_type': 'int',
+                       'numpy_type': 'Unknown'}}
+    mock_shellwidget = Mock()
+    editor = RemoteCollectionsEditorTableView(
+        None, variables, mock_shellwidget
+    )
+    index = editor.model.index(0, 0)
+    data_function = editor.delegate.make_data_function(index)
+    value = data_function()
+    mock_shellwidget.get_value.assert_called_once_with('a')
+    assert value == mock_shellwidget.get_value.return_value
+
+
 def test_create_dataframeeditor_with_correct_format(qtbot):
     df = pandas.DataFrame(['foo', 'bar'])
     editor = CollectionsEditorTableView(None, {'df': df})
@@ -276,7 +295,7 @@ def test_create_dataframeeditor_with_correct_format(qtbot):
     editor.delegate.createEditor(None, None, editor.model.index(0, 3))
     dataframe_editor = next(iter(editor.delegate._editors.values()))['editor']
     qtbot.addWidget(dataframe_editor)
-    dataframe_editor.dataModel._format == '%10d'
+    dataframe_editor.dataModel._format_spec == '10d'
 
 
 def test_collectionsmodel_with_two_ints():
@@ -333,29 +352,66 @@ def test_shows_dataframeeditor_when_editing_index(monkeypatch):
 
 
 def test_sort_numpy_numeric_collectionsmodel():
+    if parse(numpy.__version__) >= parse("2.0.0"):
+        np20 = True
+    else:
+        np20 = False
+
     var_list = [
-        numpy.float64(1e16), numpy.float64(10), numpy.float64(1),
-        numpy.float64(0.1), numpy.float64(1e-6),
-        numpy.float64(0), numpy.float64(-1e-6), numpy.float64(-1),
-        numpy.float64(-10), numpy.float64(-1e16)
-        ]
+        numpy.float64(1e16),
+        numpy.float64(10),
+        numpy.float64(1),
+        numpy.float64(0.1),
+        numpy.float64(1e-6),
+        numpy.float64(0),
+        numpy.float64(-1e-6),
+        numpy.float64(-1),
+        numpy.float64(-10),
+        numpy.float64(-1e16),
+    ]
     cm = CollectionsModel(MockParent(), var_list)
     assert cm.rowCount() == 10
     assert cm.columnCount() == 4
-    cm.sort(0)  # sort by index
-    assert data_table(cm, 10, 4) == [list(range(0, 10)),
-                                     [u'float64']*10,
-                                     [1]*10,
-                                     ['1e+16', '10.0', '1.0', '0.1',
-                                      '1e-06', '0.0', '-1e-06',
-                                      '-1.0', '-10.0', '-1e+16']]
-    cm.sort(3)  # sort by value
-    assert data_table(cm, 10, 4) == [list(range(9, -1, -1)),
-                                     [u'float64']*10,
-                                     [1]*10,
-                                     ['-1e+16', '-10.0', '-1.0',
-                                      '-1e-06', '0.0', '1e-06',
-                                      '0.1', '1.0', '10.0', '1e+16']]
+
+    # Sort by index
+    cm.sort(0)
+    assert data_table(cm, 10, 4) == [
+        list(range(0, 10)),
+        ["float64"] * 10,
+        [1] * 10,
+        [
+            "np.float64(1e+16)" if np20 else "1e+16",
+            "np.float64(10.0)" if np20 else "10.0",
+            "np.float64(1.0)" if np20 else "1.0",
+            "np.float64(0.1)" if np20 else "0.1",
+            "np.float64(1e-06)" if np20 else "1e-06",
+            "np.float64(0.0)" if np20 else "0.0",
+            "np.float64(-1e-06)" if np20 else "-1e-06",
+            "np.float64(-1.0)" if np20 else "-1.0",
+            "np.float64(-10.0)" if np20 else "-10.0",
+            "np.float64(-1e+16)" if np20 else "-1e+16",
+        ],
+    ]
+
+    # Sort by value
+    cm.sort(3)
+    assert data_table(cm, 10, 4) == [
+        list(range(9, -1, -1)),
+        ["float64"] * 10,
+        [1] * 10,
+        [
+            "np.float64(-1e+16)" if np20 else "-1e+16",
+            "np.float64(-10.0)" if np20 else "-10.0",
+            "np.float64(-1.0)" if np20 else "-1.0",
+            "np.float64(-1e-06)" if np20 else "-1e-06",
+            "np.float64(0.0)" if np20 else "0.0",
+            "np.float64(1e-06)" if np20 else "1e-06",
+            "np.float64(0.1)" if np20 else "0.1",
+            "np.float64(1.0)" if np20 else "1.0",
+            "np.float64(10.0)" if np20 else "10.0",
+            "np.float64(1e+16)" if np20 else "1e+16",
+        ],
+    ]
 
 
 def test_sort_float_collectionsmodel():
@@ -481,6 +537,91 @@ def test_rename_and_duplicate_item_in_collection_editor():
             assert editor.source_model.get_data() == coll_copy + [coll_copy[0]]
 
 
+def test_collectioneditorwidget_refresh_action_disabled():
+    """
+    Test that the Refresh button is disabled by default.
+    """
+    lst = [1, 2, 3, 4]
+    widget = CollectionsEditorWidget(None, lst.copy())
+    assert not widget.refresh_action.isEnabled()
+
+
+def test_collectioneditor_refresh():
+    """
+    Test that after pressing the refresh button, the value of the editor is
+    replaced by the return value of the data_function.
+    """
+    old_list = [1, 2, 3, 4]
+    new_list = [3, 1, 4, 1, 5]
+    editor = CollectionsEditor(None, data_function=lambda: new_list)
+    editor.setup(old_list)
+    assert editor.get_value() == old_list
+    assert editor.widget.refresh_action.isEnabled()
+    editor.widget.refresh_action.trigger()
+    assert editor.get_value() == new_list
+
+
+@pytest.mark.parametrize('result', [QMessageBox.Yes, QMessageBox.No])
+def test_collectioneditor_refresh_after_edit(result):
+    """
+    Test that after changing a value in the collections editor, refreshing the
+    editor opens a dialog box (which asks for confirmation), and that the
+    editor is only refreshed if the user clicks Yes.
+    """
+    old_list = [1, 2, 3, 4]
+    edited_list = [1, 2, 3, 5]
+    new_list = [3, 1, 4, 1, 5]
+    editor = CollectionsEditor(None, data_function=lambda: new_list)
+    editor.setup(old_list)
+    editor.show()
+    model = editor.widget.editor.source_model
+    model.setData(model.index(3, 3), '5')
+    with patch('spyder.widgets.collectionseditor.QMessageBox.question',
+               return_value=result) as mock_question:
+        editor.widget.refresh_action.trigger()
+    mock_question.assert_called_once()
+    editor.accept()
+    if result == QMessageBox.Yes:
+        assert editor.get_value() == new_list
+    else:
+        assert editor.get_value() == edited_list
+
+
+def test_collectioneditor_refresh_when_variable_deleted(qtbot):
+    """
+    Test that if the variable is deleted and then the editor is refreshed
+    (resulting in data_function raising a KeyError), a critical dialog box
+    is displayed and that the editor is closed.
+    """
+    def datafunc():
+        raise KeyError
+    lst = [1, 2, 3, 4]
+    editor = CollectionsEditor(None, data_function=datafunc)
+    editor.setup(lst)
+    with patch('spyder.widgets.collectionseditor.QMessageBox'
+               '.critical') as mock_critical, \
+         qtbot.waitSignal(editor.rejected, timeout=0):
+        editor.widget.refresh_action.trigger()
+    mock_critical.assert_called_once()
+
+
+def test_collectioneditor_refresh_nested():
+    """
+    Open an editor for a list with a tuple nested inside, and then open another
+    editor for the nested tuple. Test that refreshing the second editor works.
+    """
+    old_list = [1, 2, 3, (4, 5)]
+    new_list = [1, 2, 3, (4,)]
+    editor = CollectionsEditor(None, data_function=lambda: new_list)
+    editor.setup(old_list)
+    view = editor.widget.editor
+    view.edit(view.model.index(3, 3))
+    nested_editor = list(view.delegate._editors.values())[0]['editor']
+    assert nested_editor.get_value() == (4, 5)
+    nested_editor.widget.refresh_action.trigger()
+    assert nested_editor.get_value() == (4,)
+
+
 def test_edit_datetime(monkeypatch):
     """
     Test datetimes are editable and NaT values are correctly handled.
@@ -926,5 +1067,20 @@ def test_dicts_natural_sorting_mixed_types():
                                     ['(0,)', 2, 3, str_size]]
 
 
+def test_collectioneditor_plot(qtbot):
+    """
+    Test that plotting a list from the collection editor calls the .plot()
+    function in the associated namespace browser.
+    """
+    my_list = [4, 2]
+    mock_namespacebrowser = Mock()
+    cew = CollectionsEditorWidget(
+        None, {'list': my_list}, namespacebrowser=mock_namespacebrowser)
+    qtbot.addWidget(cew)
+
+    cew.editor.plot('list', 'plot')
+    mock_namespacebrowser.plot.assert_called_once_with(my_list, 'plot')
+
+
 if __name__ == "__main__":
     pytest.main()
